#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     August 2018                                                                     #
#                                                                                           #
#############################################################################################

# Create a window for import points from a file


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.pointset.inputoutput import inputoutput as point_io
from cognitivegeo.src.pointset.analysis import analysis as point_ays
from cognitivegeo.src.vis.messager import messager as vis_msg


QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class importpointsetfile(object):

    pointsetdata = {}
    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    #
    filelist = []

    def setupGUI(self, ImportPointSetFile):
        ImportPointSetFile.setObjectName("ImportPointSetFile")
        ImportPointSetFile.setFixedSize(400, 320)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/copy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ImportPointSetFile.setWindowIcon(icon)
        self.lblfile = QtWidgets.QLabel(ImportPointSetFile)
        self.lblfile.setObjectName("lblfile")
        self.lblfile.setGeometry(QtCore.QRect(10, 10, 110, 30))
        self.ldtfile = QtWidgets.QLineEdit(ImportPointSetFile)
        self.ldtfile.setObjectName("ldtfile")
        self.ldtfile.setGeometry(QtCore.QRect(130, 10, 190, 30))
        self.btnfile = QtWidgets.QPushButton(ImportPointSetFile)
        self.btnfile.setObjectName("btnfile")
        self.btnfile.setGeometry(QtCore.QRect(330, 10, 60, 30))
        self.lbltype = QtWidgets.QLabel(ImportPointSetFile)
        self.lbltype.setObjectName("lbltype")
        self.lbltype.setGeometry(QtCore.QRect(30, 50, 100, 30))
        self.cbbtype = QtWidgets.QComboBox(ImportPointSetFile)
        self.cbbtype.setObjectName("cbbtype")
        self.cbbtype.setGeometry(QtCore.QRect(130, 50, 260, 30))
        #
        self.lblpara = QtWidgets.QLabel(ImportPointSetFile)
        self.lblpara.setObjectName("lblpara")
        self.lblpara.setGeometry(QtCore.QRect(10, 100, 110, 30))
        self.lblinl = QtWidgets.QLabel(ImportPointSetFile)
        self.lblinl.setObjectName("lblinl")
        self.lblinl.setGeometry(QtCore.QRect(20, 140, 100, 30))
        self.cbbinl = QtWidgets.QComboBox(ImportPointSetFile)
        self.cbbinl.setObjectName("cbbinl")
        self.cbbinl.setGeometry(QtCore.QRect(130, 140, 60, 30))
        self.lblxl = QtWidgets.QLabel(ImportPointSetFile)
        self.lblxl.setObjectName("lblxl")
        self.lblxl.setGeometry(QtCore.QRect(20, 180, 100, 30))
        self.cbbxl = QtWidgets.QComboBox(ImportPointSetFile)
        self.cbbxl.setObjectName("cbbxl")
        self.cbbxl.setGeometry(QtCore.QRect(130, 180, 60, 30))
        self.lblz = QtWidgets.QLabel(ImportPointSetFile)
        self.lblz.setObjectName("lbz")
        self.lblz.setGeometry(QtCore.QRect(20, 220, 100, 30))
        self.cbbz = QtWidgets.QComboBox(ImportPointSetFile)
        self.cbbz.setObjectName("cbbz")
        self.cbbz.setGeometry(QtCore.QRect(130, 220, 60, 30))
        self.lblcomment = QtWidgets.QLabel(ImportPointSetFile)
        self.lblcomment.setObjectName("lblcomment")
        self.lblcomment.setGeometry(QtCore.QRect(220, 140, 100, 30))
        self.cbbcomment = QtWidgets.QComboBox(ImportPointSetFile)
        self.cbbcomment.setObjectName("cbbcomment")
        self.cbbcomment.setGeometry(QtCore.QRect(330, 140, 60, 30))
        self.lbldelimiter = QtWidgets.QLabel(ImportPointSetFile)
        self.lbldelimiter.setObjectName("lbldelimiter")
        self.lbldelimiter.setGeometry(QtCore.QRect(220, 180, 100, 30))
        self.cbbdelimiter = QtWidgets.QComboBox(ImportPointSetFile)
        self.cbbdelimiter.setObjectName("cbbdelimiter")
        self.cbbdelimiter.setGeometry(QtCore.QRect(330, 180, 60, 30))
        #
        self.btnimport = QtWidgets.QPushButton(ImportPointSetFile)
        self.btnimport.setObjectName("btnimport")
        self.btnimport.setGeometry(QtCore.QRect(120, 270, 160, 30))
        self.btnimport.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ImportPointSetFile)
        self.msgbox.setObjectName("msgbox")
        _center_x = ImportPointSetFile.geometry().center().x()
        _center_y = ImportPointSetFile.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ImportPointSetFile)
        QtCore.QMetaObject.connectSlotsByName(ImportPointSetFile)


    def retranslateGUI(self, ImportPointSetFile):
        self.dialog = ImportPointSetFile
        #
        _translate = QtCore.QCoreApplication.translate
        ImportPointSetFile.setWindowTitle(_translate("ImportPointSetFile", "Import PointSet from File"))
        self.lblfile.setText(_translate("ImportPointSetFile", "Select pointset files:"))
        self.lblfile.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtfile.setText(_translate("ImportPointSetFile", os.path.abspath(self.rootpath)))
        self.btnfile.setText(_translate("ImportPointSetFile", "Browse"))
        self.btnfile.clicked.connect(self.clickBtnFile)
        self.lbltype.setText(_translate("ImportPointSetFile", "\t    Type:"))
        self.cbbtype.addItems(['Kingdom 3D interpretation lines (ASCII) (*.*)',
                               'Seisworks 3D interpretation (ASCII) (*.*)',
                               'Customized (ASCII) (*.*)'])
        self.cbbtype.currentIndexChanged.connect(self.changeCbbType)
        #
        self.lblpara.setText(_translate("ImportPointSetFile", "Settings:"))
        self.lblinl.setText(_translate("ImportPointSetFile", "Inline column"))
        self.lblinl.setAlignment(QtCore.Qt.AlignRight)
        self.cbbinl.addItems([str(i+1) for i in range(10)])
        self.cbbinl.setCurrentIndex(2)
        self.cbbinl.setEnabled(False)
        self.lblxl.setText(_translate("ImportPointSetFile", "Crossline column:"))
        self.lblxl.setAlignment(QtCore.Qt.AlignRight)
        self.cbbxl.addItems([str(i + 1) for i in range(10)])
        self.cbbxl.setCurrentIndex(3)
        self.cbbxl.setEnabled(False)
        self.lblz.setText(_translate("ImportPointSetFile", "Time/depth column:"))
        self.lblz.setAlignment(QtCore.Qt.AlignRight)
        self.cbbz.addItems([str(i + 1) for i in range(10)])
        self.cbbz.setCurrentIndex(4)
        self.cbbz.setEnabled(False)
        self.lblcomment.setText(_translate("ImportPointSetFile", "Header with "))
        self.lblcomment.setAlignment(QtCore.Qt.AlignRight)
        self.cbbcomment.addItems(['None', '#', '!'])
        self.cbbcomment.setCurrentIndex(0)
        self.cbbcomment.setEnabled(False)
        self.lbldelimiter.setText(_translate("ImportPointSetFile", "Delimiter: "))
        self.lbldelimiter.setAlignment(QtCore.Qt.AlignRight)
        self.cbbdelimiter.addItems(['Space', 'Comma'])
        self.cbbdelimiter.setCurrentIndex(0)
        self.cbbdelimiter.setEnabled(False)
        #
        self.btnimport.setText(_translate("ImportPointSetFile", "Import PointSet"))
        self.btnimport.clicked.connect(self.clickBtnImportPointSetFile)


    def clickBtnFile(self):
        _dialog = QtWidgets.QFileDialog()
        _file = _dialog.getOpenFileNames(None, 'Select Point File(s)', self.rootpath,
                                         filter="All files (*.*)")
        if len(_file[0]) > 0:
            self.filelist = _file[0]
            self.ldtfile.setText(str(_file[0]))


    def changeCbbType(self):
        if self.cbbtype.currentIndex() == 0:
            self.cbbinl.setCurrentIndex(2)
            self.cbbinl.setEnabled(False)
            self.cbbxl.setCurrentIndex(3)
            self.cbbxl.setEnabled(False)
            self.cbbz.setCurrentIndex(4)
            self.cbbz.setEnabled(False)
            self.cbbcomment.setCurrentIndex(0)
            self.cbbcomment.setEnabled(False)
            self.cbbdelimiter.setCurrentIndex(0)
            self.cbbdelimiter.setEnabled(False)
        if self.cbbtype.currentIndex() == 1:
            self.cbbinl.setCurrentIndex(0)
            self.cbbinl.setEnabled(False)
            self.cbbxl.setCurrentIndex(1)
            self.cbbxl.setEnabled(False)
            self.cbbz.setCurrentIndex(4)
            self.cbbz.setEnabled(False)
            self.cbbcomment.setCurrentIndex(0)
            self.cbbcomment.setEnabled(False)
            self.cbbdelimiter.setCurrentIndex(0)
            self.cbbdelimiter.setEnabled(False)
        if self.cbbtype.currentIndex() == 2:
            self.cbbinl.setCurrentIndex(4)
            self.cbbinl.setEnabled(True)
            self.cbbxl.setCurrentIndex(3)
            self.cbbxl.setEnabled(True)
            self.cbbz.setCurrentIndex(2)
            self.cbbz.setEnabled(True)
            self.cbbcomment.setCurrentIndex(1)
            self.cbbcomment.setEnabled(True)
            self.cbbdelimiter.setCurrentIndex(0)
            self.cbbdelimiter.setEnabled(True)


    def clickBtnImportPointSetFile(self):
        self.refreshMsgBox()
        #
        _nfile = len(self.filelist)
        if _nfile <= 0:
            vis_msg.print("ERROR in ImagePointSetFile: No file selected for import", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import PointSet from File',
                                           'No file selected for import')
            return
        #
        # Progress dialog
        _pgsdlg = QtWidgets.QProgressDialog()
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/point.png")),
                       QtGui.QIcon.Normal, QtGui.QIcon.Off)
        _pgsdlg.setWindowIcon(icon)
        _pgsdlg.setWindowTitle('Import ' + str(_nfile) + ' PointSet files')
        _pgsdlg.setCancelButton(None)
        _pgsdlg.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
        _pgsdlg.forceShow()
        _pgsdlg.setFixedWidth(400)
        _pgsdlg.setMaximum(_nfile)
        #
        _pointsetdata = {}
        #
        for i in range(_nfile):
            #
            QtCore.QCoreApplication.instance().processEvents()
            #
            _filename = self.filelist[i]
            print("ImportPointSetFile: Import %d of %d pointset files: %s" % (i + 1, _nfile, _filename))
            #
            _comment = None
            if self.cbbcomment.currentIndex() == 0:
                _comment = None
            if self.cbbcomment.currentIndex() == 1:
                _comment = '#'
            if self.cbbcomment.currentIndex() == 2:
                _comment = '!'
            #
            _delimiter = None
            if self.cbbdelimiter.currentIndex() == 1:
                _delimiter = ','

            #
            _filenamemain = os.path.splitext(os.path.basename(_filename))[0]
            _pointsetdata[_filenamemain] = {}
            #
            # _data = np.loadtxt(_filename, comments=_comment)
            # #
            # if self.cbbinl.currentIndex() < np.shape(_data)[1]:
            #     _pointsetdata[_filenamemain]['Inline'] = \
            #         _data[:, self.cbbinl.currentIndex():self.cbbinl.currentIndex() + 1]
            # if self.cbbxl.currentIndex() < np.shape(_data)[1]:
            #     _pointsetdata[_filenamemain]['Crossline'] = \
            #         _data[:, self.cbbxl.currentIndex():self.cbbxl.currentIndex() + 1]
            # if self.cbbz.currentIndex() < np.shape(_data)[1]:
            #     _pointsetdata[_filenamemain]['Z'] = \
            #         _data[:, self.cbbz.currentIndex():self.cbbz.currentIndex() + 1]
            # _idx = 1
            # for _i in range(np.shape(_data)[1]):
            #     if _i != self.cbbinl.currentIndex() \
            #         and _i != self.cbbxl.currentIndex() \
            #         and _i != self.cbbz.currentIndex():
            #         _propname = 'property_' + str(_idx)
            #         _idx = _idx + 1
            #         _pointsetdata[_filenamemain][_propname] = _data[:, _i:_i+1]
            _data = point_io.readPointFromAscii(_filename, comment=_comment, delimiter=_delimiter,
                                                inlcol=self.cbbinl.currentIndex(),
                                                xlcol=self.cbbxl.currentIndex(),
                                                zcol=self.cbbz.currentIndex())
            _pointsetdata[_filenamemain]['Inline'] = _data[:, 0:1]
            _pointsetdata[_filenamemain]['Crossline'] = _data[:, 1:2]
            _pointsetdata[_filenamemain]['Z'] = _data[:, 2:3]
            for _i in range(np.shape(_data)[1]-3):
                _propname = 'property_' + str(_i+1)
                _pointsetdata[_filenamemain][_propname] = _data[:, _i+3:_i+4]
            #
            if 'Z' in _pointsetdata[_filenamemain].keys() and np.min(_pointsetdata[_filenamemain]['Z']) >= 0:
                _pointsetdata[_filenamemain]['Z'] = - _pointsetdata[_filenamemain]['Z']
            #
            _pgsdlg.setValue(i + 1)
            #
        #
        # add new data to seisdata
        for key in _pointsetdata.keys():
            if key in self.pointsetdata.keys() and checkPointData(self.pointsetdata[key]):
                reply = QtWidgets.QMessageBox.question(self.msgbox, 'Import PointSet NumPy',
                                                       key + ' already exists. Overwrite?',
                                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                       QtWidgets.QMessageBox.No)
                if reply == QtWidgets.QMessageBox.No:
                    return
            self.pointsetdata[key] = _pointsetdata[key]
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Import PointSet from File",
                                          str(_nfile) + " file(s) imported successfully")
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


def checkPointData(pointdata):
    return point_ays.checkPoint(pointdata)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    ImportPointSetFile = QtWidgets.QWidget()
    gui = importpointsetfile()
    gui.setupGUI(ImportPointSetFile)
    ImportPointSetFile.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# Create a window for editing pre-stack seismic


from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.psseismic.analysis import analysis as psseis_ays
from cognitivegeo.src.vis.messager import messager as vis_msg
from cognitivegeo.src.gui.plotvis2dpsseisshot import plotvis2dpsseisshot as gui_plotvis2dpsseisshot
from cognitivegeo.src.gui.viewpsseis import viewpsseis as gui_viewpsseis

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class editpsseis(object):

    psseisdata = {}
    plotstyle = core_set.Visual['Image']
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, EditPsSeis):
        EditPsSeis.setObjectName("EditPsSeis")
        EditPsSeis.setFixedSize(300, 420)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/psseismic.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        EditPsSeis.setWindowIcon(icon)
        #
        self.lblpsseis = QtWidgets.QLabel(EditPsSeis)
        self.lblpsseis.setObjectName("lblpsseis")
        self.lblpsseis.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.lwgpsseis = QtWidgets.QListWidget(EditPsSeis)
        self.lwgpsseis.setObjectName("lwgpsseis")
        self.lwgpsseis.setGeometry(QtCore.QRect(10, 50, 280, 200))
        self.lwgpsseis.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.lblaction = QtWidgets.QLabel(EditPsSeis)
        self.lblaction.setObjectName("lblaction")
        self.lblaction.setGeometry(QtCore.QRect(110, 270, 40, 30))
        self.cbbaction = QtWidgets.QComboBox(EditPsSeis)
        self.cbbaction.setObjectName("cbbaction")
        self.cbbaction.setGeometry(QtCore.QRect(160, 270, 130, 30))
        self.lblrename = QtWidgets.QLabel(EditPsSeis)
        self.lblrename.setObjectName("lblrename")
        self.lblrename.setGeometry(QtCore.QRect(160, 310, 40, 30))
        self.ldtrename = QtWidgets.QLineEdit(EditPsSeis)
        self.ldtrename.setObjectName("ldtrename")
        self.ldtrename.setGeometry(QtCore.QRect(200, 310, 90, 30))
        self.btnedit = QtWidgets.QPushButton(EditPsSeis)
        self.btnedit.setObjectName("btnedit")
        self.btnedit.setGeometry(QtCore.QRect(100, 370, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnedit.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(EditPsSeis)
        self.msgbox.setObjectName("msgbox")
        _center_x = EditPsSeis.geometry().center().x()
        _center_y = EditPsSeis.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(EditPsSeis)
        QtCore.QMetaObject.connectSlotsByName(EditPsSeis)


    def retranslateGUI(self, EditPsSeis):
        self.dialog = EditPsSeis
        #
        _translate = QtCore.QCoreApplication.translate
        EditPsSeis.setWindowTitle(_translate("EditPsSeis", "Edit Pre-stack Seismic"))
        self.lblpsseis.setText(_translate("EditPsSeis", "Available pre-stack seismic:"))
        self.lblaction.setText(_translate("EditPsSeis", "Action:"))
        self.cbbaction.addItems(['Copy', 'Delete', 'Rename', 'View', 'Gather-plot'])
        self.cbbaction.setItemIcon(0, QtGui.QIcon(os.path.join(self.iconpath, "icons/copy.png")))
        self.cbbaction.setItemIcon(1, QtGui.QIcon(os.path.join(self.iconpath, "icons/delete.png")))
        self.cbbaction.setItemIcon(2, QtGui.QIcon(os.path.join(self.iconpath, "icons/rename.png")))
        self.cbbaction.setItemIcon(3, QtGui.QIcon(os.path.join(self.iconpath, "icons/view.png")))
        self.cbbaction.setItemIcon(4, QtGui.QIcon(os.path.join(self.iconpath, "icons/gather.png")))
        self.cbbaction.setCurrentIndex(3)
        self.cbbaction.currentIndexChanged.connect(self.changeCbbAction)
        self.lblrename.setText(_translate("EditPsSeis", ""))
        self.lblrename.setVisible(False)
        self.ldtrename.setText(_translate("EditPsSeis", ""))
        self.ldtrename.setVisible(False)
        self.btnedit.setText(_translate("EditPsSeis", "Apply"))
        self.btnedit.clicked.connect(self.clickBtnEditPsSeis)
        #
        self.refreshLwgPsSeis()


    def changeCbbAction(self):
        if self.cbbaction.currentIndex() == 2:
            self.lblrename.setText('Name:')
            self.lblrename.setVisible(True)
            self.ldtrename.setVisible(True)
        else:
            self.lblrename.setText('')
            self.lblrename.setVisible(False)
            self.ldtrename.setText('')
            self.ldtrename.setVisible(False)


    def clickBtnEditPsSeis(self):
        self.refreshMsgBox()
        #
        _psseislist = self.lwgpsseis.selectedItems()
        if len(_psseislist) < 1:
            vis_msg.print("ERROR in EditPsSeis: No pre-stack seismic selected for editing", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Edit Pre-stack Seismic',
                                           'No pre-stack seismic selected for editing')
            return
        #
        if self.cbbaction.currentIndex() == 0:
            self.psseisdata[_psseislist[0].text()+'_copy'] = self.psseisdata[_psseislist[0].text()].copy()
        if self.cbbaction.currentIndex() == 1:
            self.psseisdata.pop(_psseislist[0].text())
        if self.cbbaction.currentIndex() == 2:
            if len(self.ldtrename.text()) < 1:
                vis_msg.print("ERROR in EditPsSeis: No name specified for rename", type='error')
                QtWidgets.QMessageBox.critical(self.msgbox,
                                               'Edit Pre-stack Seismic',
                                               'No name specified for rename')
                return
            if self.ldtrename.text() in self.psseisdata.keys():
                reply = QtWidgets.QMessageBox.question(self.msgbox, 'Edit Pre-stack Seismic',
                                                       self.ldtrename.text() + ' already exists. Overwrite?',
                                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                       QtWidgets.QMessageBox.No)

                if reply == QtWidgets.QMessageBox.No:
                    return
            self.psseisdata[self.ldtrename.text()] = self.psseisdata[_psseislist[0].text()]
            self.psseisdata.pop(_psseislist[0].text())
        if self.cbbaction.currentIndex() == 3:
            _view = QtWidgets.QDialog()
            _gui = gui_viewpsseis()
            _gui.psseisname = _psseislist[0].text()
            _gui.psseisdata = self.psseisdata[_psseislist[0].text()]
            _gui.plotstyle = self.plotstyle
            _gui.setupGUI(_view)
            _view.exec()
            _view.show()
        if self.cbbaction.currentIndex() == 4:
            _plt = QtWidgets.QDialog()
            _gui = gui_plotvis2dpsseisshot()
            _gui.psseisdata = {}
            _gui.psseisdata[_psseislist[0].text()] = self.psseisdata[_psseislist[0].text()]
            _gui.plotstyle = self.plotstyle
            _gui.setupGUI(_plt)
            _plt.exec()
            _plt.show()
        #
        self.refreshLwgPsSeis()
        # if self.cbbaction.currentIndex() != 3:
            # QtWidgets.QMessageBox.information(self.msgbox,
            #                                   "Edit PointSet",
            #                                   "PointSet edited successfully")
        return


    def refreshLwgPsSeis(self):
        self.lwgpsseis.clear()
        if len(self.psseisdata.keys()) > 0:
            for i in sorted(self.psseisdata.keys()):
                if self.checkPsSeis(i):
                    item = QtWidgets.QListWidgetItem(self.lwgpsseis)
                    item.setText(i)
                    self.lwgpsseis.addItem(item)


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkPsSeis(self, name):
        return psseis_ays.checkPsSeis(self.psseisdata[name])


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    EditPsSeis = QtWidgets.QWidget()
    gui = editpsseis()
    gui.setupGUI(EditPsSeis)
    EditPsSeis.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# Create a GUI for setting (visualization)

from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.vis.font import font as vis_font
from cognitivegeo.src.vis.line import line as vis_line
from cognitivegeo.src.vis.marker import marker as vis_marker
from cognitivegeo.src.vis.image import image as vis_image
from cognitivegeo.src.vis.colormap import colormap as vis_cmap

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class settingsvisual(object):

    settings = core_set.Visual
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, SettingsVisual):
        SettingsVisual.setObjectName("SettingsVisual")
        SettingsVisual.setFixedSize(840, 320)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/image.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        SettingsVisual.setWindowIcon(icon)
        #
        self.lblfont = QtWidgets.QLabel(SettingsVisual)
        self.lblfont.setObjectName("lblfont")
        self.lblfont.setGeometry(QtCore.QRect(430, 10, 160, 30))
        self.lblfontname = QtWidgets.QLabel(SettingsVisual)
        self.lblfontname.setObjectName("lblfontname")
        self.lblfontname.setGeometry(QtCore.QRect(450, 50, 70, 30))
        self.cbbfontname = QtWidgets.QComboBox(SettingsVisual)
        self.cbbfontname.setObjectName("cbbfontname")
        self.cbbfontname.setGeometry(QtCore.QRect(520, 50, 110, 30))
        self.lblfontweight = QtWidgets.QLabel(SettingsVisual)
        self.lblfontweight.setObjectName("lblfontweight")
        self.lblfontweight.setGeometry(QtCore.QRect(650, 90, 70, 30))
        self.cbbfontweight = QtWidgets.QComboBox(SettingsVisual)
        self.cbbfontweight.setObjectName("cbbfontweight")
        self.cbbfontweight.setGeometry(QtCore.QRect(720, 90, 110, 30))
        self.lblfontsize = QtWidgets.QLabel(SettingsVisual)
        self.lblfontsize.setObjectName("lblfontsize")
        self.lblfontsize.setGeometry(QtCore.QRect(450, 90, 70, 30))
        self.cbbfontsize = QtWidgets.QComboBox(SettingsVisual)
        self.cbbfontsize.setObjectName("cbbfontsize")
        self.cbbfontsize.setGeometry(QtCore.QRect(520, 90, 110, 30))
        self.lblfontcolor = QtWidgets.QLabel(SettingsVisual)
        self.lblfontcolor.setObjectName("lblfontcolor")
        self.lblfontcolor.setGeometry(QtCore.QRect(450, 130, 70, 30))
        self.cbbfontcolor = QtWidgets.QComboBox(SettingsVisual)
        self.cbbfontcolor.setObjectName("cbbfontcolor")
        self.cbbfontcolor.setGeometry(QtCore.QRect(520, 130, 110, 30))
        #
        self.lblline = QtWidgets.QLabel(SettingsVisual)
        self.lblline.setObjectName("lblline")
        self.lblline.setGeometry(QtCore.QRect(10, 10, 160, 30))
        self.lbllinecolor = QtWidgets.QLabel(SettingsVisual)
        self.lbllinecolor.setObjectName("lbllinecolor")
        self.lbllinecolor.setGeometry(QtCore.QRect(30, 50, 70, 30))
        self.cbblinecolor = QtWidgets.QComboBox(SettingsVisual)
        self.cbblinecolor.setObjectName("cbblinecolor")
        self.cbblinecolor.setGeometry(QtCore.QRect(100, 50, 110, 30))
        self.lbllinestyle = QtWidgets.QLabel(SettingsVisual)
        self.lbllinestyle.setObjectName("lbllinestyle")
        self.lbllinestyle.setGeometry(QtCore.QRect(30, 90, 70, 30))
        self.cbblinestyle = QtWidgets.QComboBox(SettingsVisual)
        self.cbblinestyle.setObjectName("cbblinestyle")
        self.cbblinestyle.setGeometry(QtCore.QRect(100, 90, 110, 30))
        self.lbllinewidth = QtWidgets.QLabel(SettingsVisual)
        self.lbllinewidth.setObjectName("lbllinewidth")
        self.lbllinewidth.setGeometry(QtCore.QRect(230, 90, 70, 30))
        self.cbblinewidth = QtWidgets.QComboBox(SettingsVisual)
        self.cbblinewidth.setObjectName("cbblinewidth")
        self.cbblinewidth.setGeometry(QtCore.QRect(300, 90, 110, 30))
        self.lblmarkerstyle = QtWidgets.QLabel(SettingsVisual)
        self.lblmarkerstyle.setObjectName("lblmarkerstyle")
        self.lblmarkerstyle.setGeometry(QtCore.QRect(30, 130, 70, 30))
        self.cbbmarkerstyle = QtWidgets.QComboBox(SettingsVisual)
        self.cbbmarkerstyle.setObjectName("cbbmarkerstyle")
        self.cbbmarkerstyle.setGeometry(QtCore.QRect(100, 130, 110, 30))
        self.lblmarkersize = QtWidgets.QLabel(SettingsVisual)
        self.lblmarkersize.setObjectName("lblmarkersize")
        self.lblmarkersize.setGeometry(QtCore.QRect(230, 130, 70, 30))
        self.cbbmarkersize = QtWidgets.QComboBox(SettingsVisual)
        self.cbbmarkersize.setObjectName("cbbmarkersize")
        self.cbbmarkersize.setGeometry(QtCore.QRect(300, 130, 110, 30))
        #
        self.lblsurf = QtWidgets.QLabel(SettingsVisual)
        self.lblsurf.setObjectName("lblsurf")
        self.lblsurf.setGeometry(QtCore.QRect(10, 180, 160, 30))
        self.lblcmap = QtWidgets.QLabel(SettingsVisual)
        self.lblcmap.setObjectName("lblcmap")
        self.lblcmap.setGeometry(QtCore.QRect(30, 220, 70, 30))
        self.cbbcmap = QtWidgets.QComboBox(SettingsVisual)
        self.cbbcmap.setObjectName("cbbcolor")
        self.cbbcmap.setGeometry(QtCore.QRect(100, 220, 110, 30))
        self.lblinterp = QtWidgets.QLabel(SettingsVisual)
        self.lblinterp.setObjectName("lblinterp")
        self.lblinterp.setGeometry(QtCore.QRect(230, 220, 70, 30))
        self.cbbinterp = QtWidgets.QComboBox(SettingsVisual)
        self.cbbinterp.setObjectName("cbbinterp")
        self.cbbinterp.setGeometry(QtCore.QRect(300, 220, 110, 30))
        #
        self.btnapply = QtWidgets.QPushButton(SettingsVisual)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(370, 270, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(SettingsVisual)
        self.msgbox.setObjectName("msgbox")
        _center_x = SettingsVisual.geometry().center().x()
        _center_y = SettingsVisual.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(SettingsVisual)
        QtCore.QMetaObject.connectSlotsByName(SettingsVisual)


    def retranslateGUI(self, SettingsVisual):
        self.dialog = SettingsVisual
        #
        _translate = QtCore.QCoreApplication.translate
        SettingsVisual.setWindowTitle(_translate("SettingsVisual", "Visual Settings"))
        #
        self.lblfont.setText(_translate("SettingsVisual", "Font:"))
        self.lblfontname.setText(_translate("SettingsVisual", "Name:"))
        self.cbbfontname.addItems(vis_font.FontNameList)
        self.lblfontweight.setText(_translate("SettingsVisual", "Weight:"))
        self.cbbfontweight.addItems(vis_font.FontWeightList)
        self.lblfontsize.setText(_translate("SettingsVisual", "Size:"))
        self.cbbfontsize.addItems([str(w) for w in vis_font.FontSizeList])
        self.lblfontcolor.setText(_translate("SettingsVisual", "Color:"))
        self.cbbfontcolor.addItems(vis_font.FontColorList)
        for _i in range(len(vis_font.FontColorList)):
            self.cbbfontcolor.setItemIcon(_i, QtGui.QIcon(
                QtGui.QPixmap(
                    os.path.join(self.iconpath, "icons/color_" + vis_font.FontColorList[_i] + ".png")).scaled(80, 40)))
        #
        self.lblline.setText(_translate("SettingsVisual", "Line / Curve:"))
        self.lbllinecolor.setText(_translate("SettingsVisual", "Color:"))
        self.cbblinecolor.addItems(vis_line.LineColorList)
        for _i in range(len(vis_line.LineColorList)):
            self.cbblinecolor.setItemIcon(_i, QtGui.QIcon(
                QtGui.QPixmap(
                    os.path.join(self.iconpath, "icons/color_" + vis_line.LineColorList[_i] + ".png")).scaled(80, 40)))
        self.lbllinestyle.setText(_translate("SettingsVisual", "Line style:"))
        self.cbblinestyle.addItems(vis_line.LineStyleList)
        for _i in range(len(vis_line.LineStyleList)):
            self.cbblinestyle.setItemIcon(_i, QtGui.QIcon(
                QtGui.QPixmap(
                    os.path.join(self.iconpath, "icons/line_" + vis_line.LineStyleList[_i] + ".png")).scaled(80, 40)))
        self.lbllinewidth.setText(_translate("SettingsVisual", "Line width:"))
        self.cbblinewidth.addItems([str(w) for w in vis_line.LineWidthList])
        self.lblmarkerstyle.setText(_translate("SettingsVisual", "Marker style:"))
        self.cbbmarkerstyle.addItems(vis_marker.MarkerStyleList)
        self.lblmarkersize.setText(_translate("SettingsVisual", "Marker size:"))
        self.cbbmarkersize.addItems([str(w) for w in vis_marker.MarkerSizeList])
        #
        self.lblsurf.setText(_translate("SettingsVisual", "Surface / Volume:"))
        self.lblcmap.setText(_translate("SettingsVisual", "Color map:"))
        self.cbbcmap.addItems(vis_cmap.ColorMapList)
        for _i in range(len(vis_cmap.ColorMapList)):
            self.cbbcmap.setItemIcon(_i, QtGui.QIcon(
                QtGui.QPixmap(
                    os.path.join(self.iconpath, "icons/cmap_" + vis_cmap.ColorMapList[_i] + ".png")).scaled(80, 40)))
        self.lblinterp.setText(_translate("SettingsVisual", "Interpolation:"))
        self.cbbinterp.addItems(vis_image.InterpolationList)
        for _i in range(len(vis_image.InterpolationList)):
            self.cbbinterp.setItemIcon(_i, QtGui.QIcon(
                QtGui.QPixmap(
                    os.path.join(self.iconpath, "icons/interp_" + vis_image.InterpolationList[_i] + ".png")).scaled(80, 40)))
        #
        self.btnapply.setText(_translate("SettingsVisual", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)
        #
        self.cbbfontname.setCurrentIndex(list.index(vis_font.FontNameList, self.settings['Font']['Name']))
        self.cbbfontweight.setCurrentIndex(list.index(vis_font.FontWeightList, self.settings['Font']['Weight']))
        self.cbbfontsize.setCurrentIndex(list.index(vis_font.FontSizeList, self.settings['Font']['Size']))
        self.cbbfontcolor.setCurrentIndex(list.index(vis_font.FontColorList, self.settings['Font']['Color']))
        #
        self.cbblinecolor.setCurrentIndex(list.index(vis_line.LineColorList, self.settings['Line']['Color']))
        self.cbblinewidth.setCurrentIndex(list.index(vis_line.LineWidthList, self.settings['Line']['Width']))
        self.cbblinestyle.setCurrentIndex(list.index(vis_line.LineStyleList, self.settings['Line']['Style']))
        self.cbbmarkerstyle.setCurrentIndex(list.index(vis_marker.MarkerStyleList, self.settings['Line']['MarkerStyle']))
        self.cbbmarkersize.setCurrentIndex(list.index(vis_marker.MarkerSizeList, self.settings['Line']['MarkerSize']))
        #
        self.cbbcmap.setCurrentIndex(list.index(vis_cmap.ColorMapList, self.settings['Image']['Colormap']))
        self.cbbinterp.setCurrentIndex(list.index(vis_image.InterpolationList, self.settings['Image']['Interpolation']))



    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        self.settings['Font']['Name'] = vis_font.FontNameList[self.cbbfontname.currentIndex()]
        self.settings['Font']['Weight'] = vis_font.FontWeightList[self.cbbfontweight.currentIndex()]
        self.settings['Font']['Size'] = vis_font.FontSizeList[self.cbbfontsize.currentIndex()]
        self.settings['Font']['Color'] = vis_font.FontColorList[self.cbbfontcolor.currentIndex()]
        #
        self.settings['Line']['Color'] = vis_line.LineColorList[self.cbblinecolor.currentIndex()]
        self.settings['Line']['Width'] = vis_line.LineWidthList[self.cbblinewidth.currentIndex()]
        self.settings['Line']['Style'] = vis_line.LineStyleList[self.cbblinestyle.currentIndex()]
        self.settings['Line']['MarkerStyle'] = vis_marker.MarkerStyleList[self.cbbmarkerstyle.currentIndex()]
        self.settings['Line']['MarkerSize'] = vis_marker.MarkerSizeList[self.cbbmarkersize.currentIndex()]
        #
        self.settings['Image']['Colormap'] = vis_cmap.ColorMapList[self.cbbcmap.currentIndex()]
        self.settings['Image']['Interpolation'] = vis_image.InterpolationList[self.cbbinterp.currentIndex()]
        #
        self.dialog.close()


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    SettingsVisual = QtWidgets.QWidget()
    gui = settingsvisual()
    gui.setupGUI(SettingsVisual)
    SettingsVisual.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     November 2019                                                                   #
#                                                                                           #
#############################################################################################

# Create a GUI for configuring seismic visualization

from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.vis.image import image as vis_image
from cognitivegeo.src.vis.colormap import colormap as vis_cmap

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class configseisvis(object):

    seisvisconfig = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ConfigSeisVis):
        ConfigSeisVis.setObjectName("ConfigSeisVis")
        ConfigSeisVis.setFixedSize(620, 280)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/settings.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ConfigSeisVis.setWindowIcon(icon)
        #
        self.twgseis = QtWidgets.QTableWidget(ConfigSeisVis)
        self.twgseis.setObjectName("twgseis")
        self.twgseis.setGeometry(QtCore.QRect(10, 10, 600, 200))
        self.twgseis.setColumnCount(7)
        self.twgseis.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.twgseis.verticalHeader().hide()
        self.twgseis.setColumnWidth(2, 40)
        #
        self.btnapply = QtWidgets.QPushButton(ConfigSeisVis)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(160, 230, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ConfigSeisVis)
        self.msgbox.setObjectName("msgbox")
        _center_x = ConfigSeisVis.geometry().center().x()
        _center_y = ConfigSeisVis.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ConfigSeisVis)
        QtCore.QMetaObject.connectSlotsByName(ConfigSeisVis)


    def retranslateGUI(self, ConfigSeisVis):
        self.dialog = ConfigSeisVis
        #
        _translate = QtCore.QCoreApplication.translate
        ConfigSeisVis.setWindowTitle(_translate("ConfigSeisVis", "Seismic Visualization Configuration"))
        #
        self.twgseis.clear()
        self.twgseis.setHorizontalHeaderLabels(["Seismic", "Colormap", '', 'Opacity', 'Interpolation', 'Maximum', 'Minimum'])
        self.twgseis.setRowCount(len(self.seisvisconfig.keys()))
        _idx = 0
        for seis in sorted(self.seisvisconfig.keys()):
            item = QtWidgets.QTableWidgetItem()
            item.setText(seis)
            item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgseis.setItem(_idx, 0, item)
            #
            item = QtWidgets.QComboBox()
            item.addItems(vis_cmap.ColorMapList)
            for _i in range(len(vis_cmap.ColorMapList)):
                item.setItemIcon(_i, QtGui.QIcon(
                    QtGui.QPixmap(
                        os.path.join(self.iconpath, "icons/cmap_" + vis_cmap.ColorMapList[_i] + ".png")).scaled(80,
                                                                                                                40)))
            item.setCurrentIndex(list.index(vis_cmap.ColorMapList,
                                            self.seisvisconfig[seis]['Colormap']))
            self.twgseis.setCellWidget(_idx, 1, item)
            #
            item = QtWidgets.QCheckBox()
            item.setIcon(QtGui.QIcon(QtGui.QPixmap(os.path.join(self.iconpath, "icons/flip.png")).scaled(40, 40)))
            item.setChecked(self.seisvisconfig[seis]['Flip'])
            self.twgseis.setCellWidget(_idx, 2, item)
            #
            item = QtWidgets.QComboBox()
            item.addItems(vis_cmap.OpacityList)
            item.setCurrentIndex(list.index(vis_cmap.OpacityList,
                                            self.seisvisconfig[seis]['Opacity']))
            self.twgseis.setCellWidget(_idx, 3, item)
            #
            item = QtWidgets.QComboBox()
            item.addItems(vis_image.InterpolationList)
            for _i in range(len(vis_image.InterpolationList)):
                item.setItemIcon(_i, QtGui.QIcon(
                    QtGui.QPixmap(
                        os.path.join(self.iconpath, "icons/interp_" + vis_image.InterpolationList[_i] + ".png")).scaled(
                        80, 40)))
            item.setCurrentIndex(list.index(vis_image.InterpolationList,
                                            self.seisvisconfig[seis]['Interpolation']))
            self.twgseis.setCellWidget(_idx, 4, item)
            #
            item = QtWidgets.QTableWidgetItem()
            item.setText(str(self.seisvisconfig[seis]['Maximum']))
            item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgseis.setItem(_idx, 5, item)
            #
            item = QtWidgets.QTableWidgetItem()
            item.setText(str(self.seisvisconfig[seis]['Minimum']))
            item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgseis.setItem(_idx, 6, item)
            #
            _idx = _idx + 1
        #
        self.btnapply.setText(_translate("ConfigSeisVis", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        for seis in sorted(self.seisvisconfig.keys()):
            _idx = list.index(sorted(self.seisvisconfig.keys()), seis)
            _config = {}
            _config['Colormap'] = vis_cmap.ColorMapList[self.twgseis.cellWidget(_idx, 1).currentIndex()]
            _config['Flip'] = self.twgseis.cellWidget(_idx, 2).isChecked()
            _config['Opacity'] = vis_cmap.OpacityList[self.twgseis.cellWidget(_idx, 3).currentIndex()]
            _config['Interpolation'] = vis_image.InterpolationList[self.twgseis.cellWidget(_idx, 4).currentIndex()]
            _max = basic_data.str2float(self.twgseis.item(_idx, 5).text())
            _min = basic_data.str2float(self.twgseis.item(_idx, 6).text())
            if _max is not False and _min is not False and _max >= _min:
                _config['Maximum'] = _max
                _config['Minimum'] = _min
            self.seisvisconfig[seis] = _config
        #
        self.dialog.close()


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ConfigSeisVis = QtWidgets.QWidget()
    gui = configseisvis()
    _seis = {}
    _seis['seismic'] = {}
    _seis['seismic']['Colormap'] = 'Seismic'
    _seis['seismic']['Flip'] = True
    _seis['seismic']['Opacity'] = '100%'
    _seis['seismic']['Interpolation'] = 'Bicubic'
    _seis['seismic']['Maximum'] = 100.0
    _seis['seismic']['Minimum'] = -100.0
    gui.seisvisconfig = _seis
    gui.setupGUI(ConfigSeisVis)
    ConfigSeisVis.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# Create a GUI for 1D curve plot

from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
import numpy as np
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.color import color as vis_color
from cognitivegeo.src.vis.line import line as vis_line
from cognitivegeo.src.vis.marker import marker as vis_marker
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class configlineplotting(object):

    lineplottingconfig = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ConfigLinePlotting):
        ConfigLinePlotting.setObjectName("ConfigLinePlotting")
        ConfigLinePlotting.setFixedSize(420, 280)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/settings.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ConfigLinePlotting.setWindowIcon(icon)
        #
        self.twgline = QtWidgets.QTableWidget(ConfigLinePlotting)
        self.twgline.setObjectName("twgline")
        self.twgline.setGeometry(QtCore.QRect(10, 10, 400, 200))
        self.twgline.setColumnCount(6)
        self.twgline.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.twgline.verticalHeader().hide()
        #
        self.btnapply = QtWidgets.QPushButton(ConfigLinePlotting)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(160, 230, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ConfigLinePlotting)
        self.msgbox.setObjectName("msgbox")
        _center_x = ConfigLinePlotting.geometry().center().x()
        _center_y = ConfigLinePlotting.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ConfigLinePlotting)
        QtCore.QMetaObject.connectSlotsByName(ConfigLinePlotting)


    def retranslateGUI(self, ConfigLinePlotting):
        self.dialog = ConfigLinePlotting
        #
        _translate = QtCore.QCoreApplication.translate
        ConfigLinePlotting.setWindowTitle(_translate("ConfigLinePlotting", "Line Plotting Configuration"))
        #
        self.twgline.clear()
        self.twgline.setHorizontalHeaderLabels(["Line", "Color", 'Width', 'Style', 'Marker', 'Marker Size'])
        self.twgline.setRowCount(len(self.lineplottingconfig.keys()))
        _idx = 0
        for line in sorted(self.lineplottingconfig.keys()):
            item = QtWidgets.QTableWidgetItem()
            item.setText(line)
            item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgline.setItem(_idx, 0, item)
            #
            item = QtWidgets.QComboBox()
            item.addItems(vis_color.ColorList)
            for _i in range(len(vis_color.ColorList)):
                item.setItemIcon(_i, QtGui.QIcon(
                    QtGui.QPixmap(
                        os.path.join(self.iconpath, "icons/color_" + vis_color.ColorList[_i] + ".png")).
                        scaled(80, 40)))
            item.setCurrentIndex(list.index(vis_color.ColorList,
                                            self.lineplottingconfig[line]['Color']))
            self.twgline.setCellWidget(_idx, 1, item)
            #
            item = QtWidgets.QComboBox()
            item.addItems([str(_i) for _i in vis_line.LineWidthList])
            item.setCurrentIndex(list.index(vis_line.LineWidthList,
                                            self.lineplottingconfig[line]['Width']))
            self.twgline.setCellWidget(_idx, 2, item)
            #
            item = QtWidgets.QComboBox()
            item.addItems(vis_line.LineStyleList)

            for _i in range(len(vis_line.LineStyleList)):
                item.setItemIcon(_i, QtGui.QIcon(
                    QtGui.QPixmap(
                        os.path.join(self.iconpath, "icons/line_" + vis_line.LineStyleList[_i] + ".png")).
                        scaled(80, 40)))
            item.setCurrentIndex(list.index(vis_line.LineStyleList,
                                            self.lineplottingconfig[line]['Style']))
            self.twgline.setCellWidget(_idx, 3, item)
            #
            item = QtWidgets.QComboBox()
            item.addItems(vis_marker.MarkerStyleList)
            item.setCurrentIndex(list.index(vis_marker.MarkerStyleList,
                                            self.lineplottingconfig[line]['MarkerStyle']))
            self.twgline.setCellWidget(_idx, 4, item)
            #
            item = QtWidgets.QComboBox()
            item.addItems([str(_i) for _i in vis_marker.MarkerSizeList])
            item.setCurrentIndex(list.index(vis_marker.MarkerSizeList,
                                            self.lineplottingconfig[line]['MarkerSize']))
            self.twgline.setCellWidget(_idx, 5, item)
            #
            _idx = _idx + 1
        #
        self.btnapply.setText(_translate("ConfigLinePlotting", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        for line in sorted(self.lineplottingconfig.keys()):
            _idx = list.index(sorted(self.lineplottingconfig.keys()), line)
            _config = {}
            _config['Color'] = vis_color.ColorList[self.twgline.cellWidget(_idx, 1).currentIndex()]
            _config['Width'] = vis_line.LineWidthList[self.twgline.cellWidget(_idx, 2).currentIndex()]
            _config['Style'] = vis_line.LineStyleList[self.twgline.cellWidget(_idx, 3).currentIndex()]
            _config['MarkerStyle'] = vis_marker.MarkerStyleList[self.twgline.cellWidget(_idx, 4).currentIndex()]
            _config['MarkerSize'] = vis_marker.MarkerSizeList[self.twgline.cellWidget(_idx, 5).currentIndex()]
            self.lineplottingconfig[line] = _config
        #
        self.dialog.close()


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ConfigLinePlotting = QtWidgets.QWidget()
    gui = configlineplotting()
    gui.setupGUI(ConfigLinePlotting)
    ConfigLinePlotting.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# pointset processing functions

from PyQt5 import QtCore
import sys, os
import numpy as np
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])


__all__ = ['analysis']


def checkPoint(point):
    """
    Check if a point dictionary in the correct format

    Args:
        point:  Points as a dictionary with at least the following keys: 'Inline', 'Crossline', and 'Z'

    Return:
        True or false
    """

    if len(point.keys()) < 1:
        return False
    #
    if 'Inline' not in point.keys() or 'Crossline' not in point.keys() or 'Z' not in point.keys():
            return False
    #
    return True


def checkPointSet(pointset):
    """
    Check if a pointset dictionary in the correct format

    Args:
        point:  Pointsets as a dictionary with each key representing a point dictionary.
                Each point dictionary contains at least the following keys: 'Inline', 'Crossline', and 'Z'

    Return:
        True or false
    """

    if pointset is None or len(pointset.keys()) < 1:
        return False
    for p in pointset.keys():
        if pointset[p] is None:
            return False
        if checkPoint(pointset[p]) is False:
            return False
    return True


class analysis:
    checkPoint = checkPoint
    checkPointSet = checkPointSet


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

from __future__ import absolute_import, division, print_function

__all__ = ['font', 'color', 'line', 'marker',
           'image', 'video', 'player', 'viewer3d',
           'messager']

import os, sys
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
#
from cognitivegeo.src.vis.font import font as font
from cognitivegeo.src.vis.color import color as color
from cognitivegeo.src.vis.line import line as line
from cognitivegeo.src.vis.marker import marker as marker
from cognitivegeo.src.vis.colormap import colormap as cmap
from cognitivegeo.src.vis.image import image as image
from cognitivegeo.src.vis.video import video as video
from cognitivegeo.src.vis.player import player as player
from cognitivegeo.src.vis.viewer3d import viewer3d as viewer3d
from cognitivegeo.src.vis.messager import messager as messager


# A mapping from data sampel format codes to SEG Y types.
DATA_SAMPLE_FORMAT_TO_SEG_Y_TYPE = {1: 'ibm',
                      2: 'int32',
                      3: 'int16',
                      5: 'float32',
                      8: 'int8'}

# A mapping from SEG Y data types to format characters used by the struct module
SEG_Y_TYPE_TO_CTYPE = {'int32':  'i',
          'uint32': 'I',
          'int16':  'h',
          'uint16': 'H',
          'int8': 'b',
          'uint8': 'B',
          'float32':  'f',
          'ibm': 'ibm'}


SEG_Y_TYPE_DESCRIPTION = {'ibm': 'IBM float',
                     'int32': '32 bit signed integer',
                     'uint32': '32 bit unsigned integer',
                     'int16': '16 bit signed integer',
                     'uint16': '16 bit unsigned integer',
                     'float32': 'IEEE float32',
                     'int8': '8 bit signed integer (byte)',
                     'uint8': '8 bit unsigned integer (byte)'}


CTYPE_TO_SIZE = dict(i=4,
             I=4,
             h=2,
             H=2,
             b=1,
             B=1,
             f=4,
             ibm=4)


def size_in_bytes(ctype):
    return CTYPE_TO_SIZE[ctype]



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     April 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for viewing seg-y textual header


from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.inputoutput import inputoutput as seis_io

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class viewsegytextualheader(object):

    segyfile = ""
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ViewSegyTextualHeader):
        ViewSegyTextualHeader.setObjectName("ViewSegyTextualHeader")
        ViewSegyTextualHeader.setFixedSize(560, 460)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/segy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ViewSegyTextualHeader.setWindowIcon(icon)
        #
        self.lblhelp = QtWidgets.QLabel(ViewSegyTextualHeader)
        self.lblhelp.setObjectName("lblhelp")
        self.lblhelp.setGeometry(QtCore.QRect(460, 10, 80, 30))
        self.lwgheader = QtWidgets.QListWidget(ViewSegyTextualHeader)
        self.lwgheader.setObjectName("lwgheader")
        self.lwgheader.setGeometry(QtCore.QRect(10, 50, 530, 380))
        self.lwgheader.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        #
        self.msgbox = QtWidgets.QMessageBox(ViewSegyTextualHeader)
        self.msgbox.setObjectName("msgbox")
        _center_x = ViewSegyTextualHeader.geometry().center().x()
        _center_y = ViewSegyTextualHeader.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ViewSegyTextualHeader)
        QtCore.QMetaObject.connectSlotsByName(ViewSegyTextualHeader)



    def retranslateGUI(self, ViewSegyTextualHeader):
        self.dialog = ViewSegyTextualHeader
        #
        _translate = QtCore.QCoreApplication.translate
        ViewSegyTextualHeader.setWindowTitle(_translate("ViewSegyTextualHeader", "SEG-Y Textual Header"))
        self.lblhelp.setText(_translate("ViewSegyTextualHeader",
                                        '<a href="https://seg.org/Portals/0/SEG/News%20and%20Resources/Technical%20Standards/seg_y_rev1.pdf">SEG-Y Format</a>'))
        self.lblhelp.setOpenExternalLinks(True)
        if os.path.exists(self.segyfile):
            _header = seis_io.readSegyTextualHeader(self.segyfile)
            for _i in _header:
                item = QtWidgets.QListWidgetItem(self.lwgheader)
                item.setText(_translate("ViewSegyTextualHeader", _i))
                self.lwgheader.addItem(item)


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ViewSegyTextualHeader = QtWidgets.QWidget()
    gui = viewsegytextualheader()
    gui.setupGUI(ViewSegyTextualHeader)
    ViewSegyTextualHeader.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for editing seismic/pointset properties


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class calculator(object):

    data = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, Calculator):
        Calculator.setObjectName("Calculator")
        Calculator.setFixedSize(250, 140)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/math.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        Calculator.setWindowIcon(icon)
        #
        self.lbloperator = QtWidgets.QLabel(Calculator)
        self.lbloperator.setObjectName("lbloperator")
        self.lbloperator.setGeometry(QtCore.QRect(10, 10, 80, 30))
        self.cbboperator = QtWidgets.QComboBox(Calculator)
        self.cbboperator.setObjectName("cbboperator")
        self.cbboperator.setGeometry(QtCore.QRect(100, 10, 140, 30))
        self.lblvalue = QtWidgets.QLabel(Calculator)
        self.lblvalue.setObjectName("lblvalue")
        self.lblvalue.setGeometry(QtCore.QRect(100, 50, 60, 30))
        self.ldtvalue = QtWidgets.QLineEdit(Calculator)
        self.ldtvalue.setObjectName("ldtvalue")
        self.ldtvalue.setGeometry(QtCore.QRect(170, 50, 70, 30))
        self.btnapply = QtWidgets.QPushButton(Calculator)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(75, 90, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(Calculator)
        self.msgbox.setObjectName("msgbox")
        _center_x = Calculator.geometry().center().x()
        _center_y = Calculator.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(Calculator)
        QtCore.QMetaObject.connectSlotsByName(Calculator)


    def retranslateGUI(self, Calculator):
        self.dialog = Calculator
        #
        _translate = QtCore.QCoreApplication.translate
        Calculator.setWindowTitle(_translate("Calculator", "Calculator"))
        self.lbloperator.setText(_translate("Calculator", "Select operator:"))
        self.cbboperator.addItems(['Assign (x=v)', 'Add (x+v)', 'Multipy (x*v)', 'Power(x^v)',
                                   'Inverse (1/x)', 'Opposite (-x)', 'Absolute(x)', 'Round(x)',
                                   'Exponential (e^x)', 'Logarithm (log.x)'])
        self.cbboperator.currentIndexChanged.connect(self.changeCbbOperator)
        self.lblvalue.setText(_translate("Calculator", "Value (v) ="))
        self.ldtvalue.setText(_translate("Calculator", ""))
        self.ldtvalue.setVisible(True)
        self.btnapply.setText(_translate("Calculator", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)


    def changeCbbOperator(self):
        if self.cbboperator.currentIndex() <= 3:
            self.lblvalue.setVisible(True)
            self.ldtvalue.setVisible(True)
        else:
            self.lblvalue.setVisible(False)
            self.ldtvalue.setText('')
            self.ldtvalue.setVisible(False)


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        if type(self.data) is not np.ndarray or np.shape(self.data)[0] < 1:
            vis_msg.print("ERROR in Calculator: No NumPy data added into calculator", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Calculator',
                                           'No NumPy data added into calculator')
            return
        #
        if self.cbboperator.currentIndex() < 4:
            _value = basic_data.str2float(self.ldtvalue.text())
            if _value is False:
                vis_msg.print("ERROR in Calculator: Non-float value", type='error')
                QtWidgets.QMessageBox.critical(self.msgbox,
                                               'Calculator',
                                               'Non-float value')
                return
        #
        if self.cbboperator.currentIndex() == 0:
            self.data = self.data * 0.0 + _value
        if self.cbboperator.currentIndex() == 1:
            self.data = self.data + _value
        if self.cbboperator.currentIndex() == 2:
            self.data = self.data * _value
        if self.cbboperator.currentIndex() == 3:
            self.data = self.data ** _value
        if self.cbboperator.currentIndex() == 4:
            self.data = np.reciprocal(self.data)
        if self.cbboperator.currentIndex() == 5:
            self.data = - self.data
        if self.cbboperator.currentIndex() == 6:
            self.data = np.abs(self.data)
        if self.cbboperator.currentIndex() == 7:
            self.data = np.round(self.data)
        if self.cbboperator.currentIndex() == 8:
            self.data = np.exp(self.data)
        if self.cbboperator.currentIndex() == 9:
            self.data = np.log(self.data)
        #
        # QtWidgets.QMessageBox.information(self.msgbox,
        #                                   "Calculator",
        #                                   "Data calculated successfully")
        self.dialog.close()
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Calculator = QtWidgets.QWidget()
    gui = calculator()
    gui.setupGUI(Calculator)
    Calculator.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     November 2019                                                                   #
#                                                                                           #
#############################################################################################

# Create a window for plot seismic inline-crossline-z slices in 3D


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.seismic.visualization import visualization as seis_vis
from cognitivegeo.src.vis.colormap import colormap as vis_cmap
from cognitivegeo.src.gui.configplayer import configplayer as gui_configplayer
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class plotvis3dseisinlxlz(object):

    survinfo = {}
    seisdata = {}
    plotstyle = core_set.Visual['Image']
    fontstyle = core_set.Visual['Font']
    playerconfiginl = core_set.Viewer['Player']
    playerconfigxl = core_set.Viewer['Player']
    playerconfigz = core_set.Viewer['Player']
    viewerconfig = core_set.Viewer['Viewer3D']['ViewFrom']
    viewerconfig['Home'] = core_set.Viewer['Viewer3D']['GoHome']
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    vispyscene = None

    def setupGUI(self, PlotVis3DSeisInlXlZ):
        PlotVis3DSeisInlXlZ.setObjectName("PlotVis3DSeisInlXlZ")
        PlotVis3DSeisInlXlZ.setFixedSize(500, 490)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/box.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        PlotVis3DSeisInlXlZ.setWindowIcon(icon)
        #
        self.lwgattrib = QtWidgets.QListWidget(PlotVis3DSeisInlXlZ)
        self.lwgattrib.setObjectName("lwgattrib")
        self.lwgattrib.setGeometry(QtCore.QRect(10, 10, 230, 410))
        self.lwgattrib.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        #
        self.lblinlslice = QtWidgets.QLabel(PlotVis3DSeisInlXlZ)
        self.lblinlslice.setObjectName("lblinlslice")
        self.lblinlslice.setGeometry(QtCore.QRect(260, 50, 150, 30))
        self.slbinlslice = QtWidgets.QScrollBar(PlotVis3DSeisInlXlZ)
        self.slbinlslice.setObjectName("slbinlslice")
        self.slbinlslice.setOrientation(QtCore.Qt.Horizontal)
        self.slbinlslice.setGeometry(QtCore.QRect(290, 80, 140, 30))
        self.ldtinlslice = QtWidgets.QLineEdit(PlotVis3DSeisInlXlZ)
        self.ldtinlslice.setObjectName("ldtinlslice")
        self.ldtinlslice.setGeometry(QtCore.QRect(440, 80, 50, 30))
        self.ldtinlslice.setAlignment(QtCore.Qt.AlignCenter)
        self.cbxinlslice = QtWidgets.QCheckBox(PlotVis3DSeisInlXlZ)
        self.cbxinlslice.setObjectName("cbxinlslice")
        self.cbxinlslice.setGeometry(QtCore.QRect(260, 80, 30, 30))
        #
        self.lblxlslice = QtWidgets.QLabel(PlotVis3DSeisInlXlZ)
        self.lblxlslice.setObjectName("lblxlslice")
        self.lblxlslice.setGeometry(QtCore.QRect(260, 120, 150, 30))
        self.slbxlslice = QtWidgets.QScrollBar(PlotVis3DSeisInlXlZ)
        self.slbxlslice.setObjectName("slbxlslice")
        self.slbxlslice.setOrientation(QtCore.Qt.Horizontal)
        self.slbxlslice.setGeometry(QtCore.QRect(290, 150, 140, 30))
        self.ldtxlslice = QtWidgets.QLineEdit(PlotVis3DSeisInlXlZ)
        self.ldtxlslice.setObjectName("ldtxlslice")
        self.ldtxlslice.setGeometry(QtCore.QRect(440, 150, 50, 30))
        self.ldtxlslice.setAlignment(QtCore.Qt.AlignCenter)
        self.cbxxlslice = QtWidgets.QCheckBox(PlotVis3DSeisInlXlZ)
        self.cbxxlslice.setObjectName("cbxxlslice")
        self.cbxxlslice.setGeometry(QtCore.QRect(260, 150, 30, 30))
        #
        self.lblzslice = QtWidgets.QLabel(PlotVis3DSeisInlXlZ)
        self.lblzslice.setObjectName("lblzslice")
        self.lblzslice.setGeometry(QtCore.QRect(260, 190, 150, 30))
        self.slbzslice = QtWidgets.QScrollBar(PlotVis3DSeisInlXlZ)
        self.slbzslice.setObjectName("slbzslice")
        self.slbzslice.setOrientation(QtCore.Qt.Horizontal)
        self.slbzslice.setGeometry(QtCore.QRect(290, 220, 140, 30))
        self.ldtzslice = QtWidgets.QLineEdit(PlotVis3DSeisInlXlZ)
        self.ldtzslice.setObjectName("ldtzslice")
        self.ldtzslice.setGeometry(QtCore.QRect(440, 220, 50, 30))
        self.ldtzslice.setAlignment(QtCore.Qt.AlignCenter)
        self.cbxzslice = QtWidgets.QCheckBox(PlotVis3DSeisInlXlZ)
        self.cbxzslice.setObjectName("cbxzslice")
        self.cbxzslice.setGeometry(QtCore.QRect(260, 220, 30, 30))
        #
        self.lblcmap = QtWidgets.QLabel(PlotVis3DSeisInlXlZ)
        self.lblcmap.setObjectName("lblcmap")
        self.lblcmap.setGeometry(QtCore.QRect(260, 350, 60, 30))
        self.cbbcmap = QtWidgets.QComboBox(PlotVis3DSeisInlXlZ)
        self.cbbcmap.setObjectName("cbbcmap")
        self.cbbcmap.setGeometry(QtCore.QRect(320, 350, 120, 30))
        self.cbxflip = QtWidgets.QCheckBox(PlotVis3DSeisInlXlZ)
        self.cbxflip.setObjectName("cbxflip")
        self.cbxflip.setGeometry(QtCore.QRect(450, 350, 40, 30))
        #
        self.lblrange = QtWidgets.QLabel(PlotVis3DSeisInlXlZ)
        self.lblrange.setObjectName("lblrange")
        self.lblrange.setGeometry(QtCore.QRect(260, 270, 40, 30))
        self.ldtmin = QtWidgets.QLineEdit(PlotVis3DSeisInlXlZ)
        self.ldtmin.setObjectName("ldtmin")
        self.ldtmin.setGeometry(QtCore.QRect(300, 270, 70, 30))
        self.ldtmin.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtmax = QtWidgets.QLineEdit(PlotVis3DSeisInlXlZ)
        self.ldtmax.setObjectName("ldtmax")
        self.ldtmax.setGeometry(QtCore.QRect(420, 270, 70, 30))
        self.ldtmax.setAlignment(QtCore.Qt.AlignCenter)
        self.lblrangeto = QtWidgets.QLabel(PlotVis3DSeisInlXlZ)
        self.lblrangeto.setObjectName("lblrangeto")
        self.lblrangeto.setGeometry(QtCore.QRect(370, 270, 50, 30))
        self.lblrangeto.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.lblzscale = QtWidgets.QLabel(PlotVis3DSeisInlXlZ)
        self.lblzscale.setObjectName("lblzscale")
        self.lblzscale.setGeometry(QtCore.QRect(260, 390, 40, 30))
        self.ldtzscale = QtWidgets.QLineEdit(PlotVis3DSeisInlXlZ)
        self.ldtzscale.setObjectName("ldtzscale")
        self.ldtzscale.setGeometry(QtCore.QRect(300, 390, 50, 30))
        self.ldtzscale.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.cbxsrvbox = QtWidgets.QCheckBox(PlotVis3DSeisInlXlZ)
        self.cbxsrvbox.setObjectName("cbxsrvbox")
        self.cbxsrvbox.setGeometry(QtCore.QRect(400, 390, 90, 30))

        #
        self.btnconfigplayer = QtWidgets.QPushButton(PlotVis3DSeisInlXlZ)
        self.btnconfigplayer.setObjectName("btnconfigplayer")
        self.btnconfigplayer.setGeometry(QtCore.QRect(390, 10, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/video.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnconfigplayer.setIcon(icon)
        #
        self.btnplotslice = QtWidgets.QPushButton(PlotVis3DSeisInlXlZ)
        self.btnplotslice.setObjectName("btnplotslice")
        self.btnplotslice.setGeometry(QtCore.QRect(170, 440, 160, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/box.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnplotslice.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(PlotVis3DSeisInlXlZ)
        self.msgbox.setObjectName("msgbox")
        _center_x = PlotVis3DSeisInlXlZ.geometry().center().x()
        _center_y = PlotVis3DSeisInlXlZ.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(PlotVis3DSeisInlXlZ)
        QtCore.QMetaObject.connectSlotsByName(PlotVis3DSeisInlXlZ)



    def retranslateGUI(self, PlotVis3DSeisInlXlZ):
        self.dialog = PlotVis3DSeisInlXlZ
        #
        _translate = QtCore.QCoreApplication.translate
        PlotVis3DSeisInlXlZ.setWindowTitle(_translate("PlotVis3DSeisInlXlZ", "3D Window: Seismic"))
        if self.checkSurvInfo() is True:
            _firstattrib = None
            for i in sorted(self.seisdata.keys()):
                if self.checkSeisData(i):
                    item = QtWidgets.QListWidgetItem(self.lwgattrib)
                    item.setText(_translate("PlotVis3DSeisInlXlZ", i))
                    self.lwgattrib.addItem(item)
                    if _firstattrib is None:
                        _firstattrib = item
            self.lwgattrib.setCurrentItem(_firstattrib)
        self.lwgattrib.itemSelectionChanged.connect(self.changeLwgAttrib)
        #
        self.lblinlslice.setText(_translate("PlotVis3DSeisInlXlZ", "@ inline:"))
        if (self.checkSurvInfo() is True):
            _slices = self.survinfo['ILRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
        else:
            _slicemin = 0
            _slicemax = 0
        self.slbinlslice.setMinimum(_slicemin)
        self.slbinlslice.setMaximum(_slicemax)
        self.slbinlslice.setValue(_slicemin)
        if (self.checkSurvInfo() is True):
            self.ldtinlslice.setText(_translate("PlotVis3DSeisInlXlZ", str(_slicemin)))
        else:
            self.ldtinlslice.setText(_translate("PlotVis3DSeisInlXlZ", ''))
        self.slbinlslice.valueChanged.connect(self.changeSlbInlSlice)
        self.ldtinlslice.textChanged.connect(self.changeLdtInlSlice)
        self.cbxinlslice.setChecked(True)
        self.cbxinlslice.stateChanged.connect(self.changeCbxInlSlice)
        #
        self.lblxlslice.setText(_translate("PlotVis3DSeisInlXlZ", "@ crossline:"))
        if (self.checkSurvInfo() is True):
            _slices = self.survinfo['XLRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
        else:
            _slicemin = 0
            _slicemax = 0
        self.slbxlslice.setMinimum(_slicemin)
        self.slbxlslice.setMaximum(_slicemax)
        self.slbxlslice.setValue(_slicemin)
        if (self.checkSurvInfo() is True):
            self.ldtxlslice.setText(_translate("PlotVis3DSeisInlXlZ", str(_slicemin)))
        else:
            self.ldtxlslice.setText(_translate("PlotVis3DSeisInlXlZ", ''))
        self.slbxlslice.valueChanged.connect(self.changeSlbXlSlice)
        self.ldtxlslice.textChanged.connect(self.changeLdtXlSlice)
        self.cbxxlslice.setChecked(True)
        self.cbxxlslice.stateChanged.connect(self.changeCbxXlSlice)
        #
        self.lblzslice.setText(_translate("PlotVis3DSeisInlXlZ", "@ time/depth:"))
        if (self.checkSurvInfo() is True):
            _slices = self.survinfo['ZRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
        else:
            _slicemin = 0
            _slicemax = 0
        self.slbzslice.setMinimum(_slicemin)
        self.slbzslice.setMaximum(_slicemax)
        self.slbzslice.setValue(_slicemin)
        if (self.checkSurvInfo() is True):
            self.ldtzslice.setText(_translate("PlotVis3DSeisInlXlZ", str(_slicemin)))
        else:
            self.ldtzslice.setText(_translate("PlotVis3DSeisInlXlZ", ''))
        self.slbzslice.valueChanged.connect(self.changeSlbZSlice)
        self.ldtzslice.textChanged.connect(self.changeLdtZSlice)
        self.cbxzslice.setChecked(True)
        self.cbxzslice.stateChanged.connect(self.changeCbxZSlice)
        #
        self.lblcmap.setText(_translate("Plot2DXlSlice", "Color map:"))
        self.cbbcmap.addItems(vis_cmap.ColorMapList)
        for _i in range(len(vis_cmap.ColorMapList)):
            self.cbbcmap.setItemIcon(_i, QtGui.QIcon(
                QtGui.QPixmap(os.path.join(self.iconpath, "icons/cmap_" + vis_cmap.ColorMapList[_i] + ".png")).scaled(80, 30)))
        self.cbbcmap.setCurrentIndex(list.index(vis_cmap.ColorMapList, self.plotstyle['Colormap']))
        #
        self.cbxflip.setText(_translate("PlotVis3DSeisInlXlZ", ""))
        self.cbxflip.setIcon(QtGui.QIcon(
            QtGui.QPixmap(os.path.join(self.iconpath, "icons/flip.png")).scaled(80, 80)))
        #
        self.lblrange.setText(_translate("PlotVis3DSeisInlXlZ", "Range:"))
        self.lblrangeto.setText(_translate("PlotVis3DSeisInlXlZ", "~~~"))
        if (self.checkSurvInfo() is True) \
                and (self.lwgattrib.currentItem() is not None) \
                and (self.checkSeisData(self.lwgattrib.currentItem().text()) is True):
            _min, _max = self.getAttribRange(self.lwgattrib.currentItem().text())
            self.ldtmin.setText(_translate("PlotVis3DSeisInlXlZ", str(_min)))
            self.ldtmax.setText(_translate("PlotVis3DSeisInlXlZ", str(_max)))
        #
        self.lblzscale.setText(_translate("PlotVis3DSeisInlXlZ", "Z-scale:"))
        self.ldtzscale.setText(_translate("PlotVis3DSeisInlXlZ", '1.0'))
        #
        self.cbxsrvbox.setText(_translate("PlotVis3DSeisInlXlZ", "Survey box"))
        self.cbxsrvbox.setChecked(True)
        #
        self.btnconfigplayer.setText(_translate("PlotVis3DSeisInlXlZ", "Slice Player"))
        self.btnconfigplayer.clicked.connect(self.clickBtnConfigPlayer)
        #
        self.btnplotslice.setText(_translate("PlotVis3DSeisInlXlZ", "3D Seismic Viewer"))
        self.btnplotslice.setDefault(True)
        self.btnplotslice.clicked.connect(self.clickBtnPlotSlice)

        self.dialog.show()


    def changeLwgAttrib(self):
        if len(self.lwgattrib.selectedItems()) > 0:
            # if len(self.lwgattrib.selectedItems()) > 1:
            #     self.ldtmin.setEnabled(False)
            #     self.ldtmax.setEnabled(False)
            # else:
            #     self.ldtmin.setEnabled(True)
            #     self.ldtmax.setEnabled(True)
            _slices = self.survinfo['ILRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
            self.slbinlslice.setMinimum(_slicemin)
            self.slbinlslice.setMaximum(_slicemax)
            self.ldtinlslice.setText(str(self.slbinlslice.value()))
            #
            _slices = self.survinfo['XLRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
            self.slbxlslice.setMinimum(_slicemin)
            self.slbxlslice.setMaximum(_slicemax)
            self.ldtxlslice.setText(str(self.slbxlslice.value()))
            #
            _slices = self.survinfo['ZRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
            self.slbzslice.setMinimum(_slicemin)
            self.slbzslice.setMaximum(_slicemax)
            self.ldtzslice.setText(str(self.slbzslice.value()))
            #
            _min, _max = self.getAttribRange(self.lwgattrib.currentItem().text())
            self.ldtmin.setText(str(_min))
            self.ldtmax.setText(str(_max))
        else:
            self.slbinlslice.setMinimum(0)
            self.slbinlslice.setMaximum(0)
            self.ldtinlslice.setText('')
            #
            self.slbxlslice.setMinimum(0)
            self.slbxlslice.setMaximum(0)
            self.ldtxlslice.setText('')
            #
            self.slbzslice.setMinimum(0)
            self.slbzslice.setMaximum(0)
            self.ldtzslice.setText('')
            #
            self.ldtmin.setText('')
            self.ldtmax.setText('')


    def changeSlbInlSlice(self):
        self.ldtinlslice.setText(str(self.slbinlslice.value()))


    def changeLdtInlSlice(self):
        if len(self.ldtinlslice.text()) > 0:
            _val = basic_data.str2int(self.ldtinlslice.text())
            if _val >= self.slbinlslice.minimum() and _val <= self.slbinlslice.maximum():
                self.slbinlslice.setValue(_val)


    def changeCbxInlSlice(self):
        if self.cbxinlslice.isChecked():
            self.ldtinlslice.setEnabled(True)
            self.slbinlslice.setEnabled(True)
        else:
            self.ldtinlslice.setEnabled(False)
            self.slbinlslice.setEnabled(False)
    
    
    def changeSlbXlSlice(self):
        self.ldtxlslice.setText(str(self.slbxlslice.value()))


    def changeLdtXlSlice(self):
        if len(self.ldtxlslice.text()) > 0:
            _val = basic_data.str2int(self.ldtxlslice.text())
            if _val >= self.slbxlslice.minimum() and _val <= self.slbxlslice.maximum():
                self.slbxlslice.setValue(_val)


    def changeCbxXlSlice(self):
        if self.cbxxlslice.isChecked():
            self.ldtxlslice.setEnabled(True)
            self.slbxlslice.setEnabled(True)
        else:
            self.ldtxlslice.setEnabled(False)
            self.slbxlslice.setEnabled(False)
    
    
    def changeSlbZSlice(self):
        self.ldtzslice.setText(str(self.slbzslice.value()))


    def changeLdtZSlice(self):
        if len(self.ldtzslice.text()) > 0:
            _val = basic_data.str2int(self.ldtzslice.text())
            if _val >= self.slbzslice.minimum() and _val <= self.slbzslice.maximum():
                self.slbzslice.setValue(_val)


    def changeCbxZSlice(self):
        if self.cbxzslice.isChecked():
            self.ldtzslice.setEnabled(True)
            self.slbzslice.setEnabled(True)
        else:
            self.ldtzslice.setEnabled(False)
            self.slbzslice.setEnabled(False)


    def clickBtnConfigPlayer(self):
        _config = QtWidgets.QDialog()
        _gui = gui_configplayer()
        _gui.playerconfig = {}
        _gui.playerconfig['Inline'] = self.playerconfiginl
        _gui.playerconfig['Crossline'] = self.playerconfigxl
        _gui.playerconfig['Time/depth'] = self.playerconfigz
        _gui.setupGUI(_config)
        _config.exec()
        self.playerconfiginl = _gui.playerconfig['Inline']
        self.playerconfigxl = _gui.playerconfig['Crossline']
        self.playerconfigz = _gui.playerconfig['Time/depth']
        _config.show()


    def clickBtnPlotSlice(self):
        self.refreshMsgBox()
        #
        _attriblist = self.lwgattrib.selectedItems()
        if len(_attriblist) < 1:
            vis_msg.print("ERROR in PlotVis3DSeisInlXlZ: No property selected for plot", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '3D Window: Seismic',
                                           'No property selected for plot')
            return
        #
        _inlsls = None
        if self.cbxinlslice.isChecked():
            _inlsls = self.slbinlslice.value()
        _xlsls = None
        if self.cbxxlslice.isChecked():
            _xlsls = self.slbxlslice.value()
        _zsls = None
        if self.cbxzslice.isChecked():
            _zsls = self.slbzslice.value()
        #
        _cmap = self.cbbcmap.currentIndex()
        _flip = self.cbxflip.isChecked()
        _min = basic_data.str2float(self.ldtmin.text())
        _max = basic_data.str2float(self.ldtmax.text())
        if _min is False or _max is False:
            vis_msg.print("ERROR in PlotVis3DSeisInlXlZ: Non-float range specified for plot", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '3D Window: Seismic',
                                           'Non-float range specified for plot')
            return

        _zscale = basic_data.str2float(self.ldtzscale.text())
        if _zsls is False or _zscale <= 0.0:
            vis_msg.print("ERROR in PlotVis3DSeisInlXlZ: z-scale > 0.0 required", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '3D Window: Seismic',
                                           'z-scale > 0.0 required')
            return
        #
        _srvbox = self.cbxsrvbox.isChecked()
        #
        for i in range(len(_attriblist)):
            print("PlotVis3DSeisInlXlZ: Plot %d of %d properties: %s" % (i + 1, len(_attriblist), _attriblist[i].text()))
            _data = self.seisdata[_attriblist[i].text()]
            seis_vis.plotSeisILXLZSlicePlayerFrom3DMat(_data, seisinfo=self.survinfo,
                                                       initinlsl=_inlsls, initxlsl=_xlsls, initzsl=_zsls,
                                                       valuemax=_max, valuemin=_min, zscale=_zscale,
                                                       colormap=vis_cmap.ColorMapList[_cmap], flipcmap=_flip,
                                                       surveyboxon=_srvbox,
                                                       interpolation=self.plotstyle['Interpolation'].lower(),
                                                       viewerconfig=self.viewerconfig,
                                                       playerconfiginl=self.playerconfiginl,
                                                       playerconfigxl=self.playerconfigxl,
                                                       playerconfigz=self.playerconfigz,
                                                       qicon=QtGui.QIcon(os.path.join(self.iconpath, "icons/logo.png")),
                                                       )
        #
        # QtWidgets.QMessageBox.information(self.msgbox,
        #                                   "Plot Inline",
        #                                   str(len(_attriblist)) + " properties plotted successfully")
        self.dialog.close()
        return


    def getAttribRange(self, f):
        _min = -1
        _max = 1
        if (self.checkSurvInfo() is True) \
                and (f in self.seisdata.keys()) \
                and (self.checkSeisData(f) is True):
            _min = np.min(self.seisdata[f])
            _max = np.max(self.seisdata[f])
        return _min, _max


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("PlotVis3DSeisInlXlZ: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Plot Seismic Inline',
            #                                'Survey not found')
            return False
        return True


    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    PlotVis3DSeisInlXlZ = QtWidgets.QWidget()
    gui = plotvis3dseisinlxlz()
    gui.setupGUI(PlotVis3DSeisInlXlZ)
    PlotVis3DSeisInlXlZ.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# basic functions for setting

import numpy as np
import os, sys


__all__ = ['settings']

# Interface settings as a dictionary with the following keys:
# 'Toolbar' ----- Toolbar availability, as a dictionary
GUI = {}
GUI['Toolbar'] = {}
GUI['Toolbar']['Left'] = True
GUI['Toolbar']['Right'] = True
GUI['Toolbar']['Top'] = True
GUI['Toolbar']['Bottom'] = True

# General settings as a dictionary with the following keys:
# 'RootPath' ----- Main path for I/O, as a string
General = {}
General['RootPath'] = os.path.dirname(__file__)[:-15]

# Visual settings as a dictionary with the following keys:
# 'Font' ----- Font style, as a dictionary
# 'Line' ----- Line style, as a dictionary
# 'Image' ----- Image style, as a dictionary
Visual = {}
Visual['Font'] = {}
Visual['Font']['Name'] = 'Times New Roman'
Visual['Font']['Color'] = 'Green'
Visual['Font']['Style'] = 'Normal'
Visual['Font']['Weight'] = 'Normal'
Visual['Font']['Size'] = 16
#
Visual['Line'] = {}
Visual['Line']['Color'] = 'Red'
Visual['Line']['Width'] = 3
Visual['Line']['Style'] = 'Solid'
Visual['Line']['MarkerStyle'] = 'None'
Visual['Line']['MarkerSize'] = 5
#
Visual['Image'] = {}
Visual['Image']['Colormap'] = 'Red-White-Blue'
Visual['Image']['Interpolation'] = 'Quadric'

# Viewer settings as a dictionary with the following keys:
# 'Viewer3D' ----- 3D viewer settings, as a dictionary
# 'Player' ----- Player settings, as a dictionary
Viewer = {}
Viewer['Viewer3D'] = {}
Viewer['Viewer3D']['GoHome'] = 'U'
Viewer['Viewer3D']['ViewFrom'] = {}
Viewer['Viewer3D']['ViewFrom']['Inline'] = 'I'
Viewer['Viewer3D']['ViewFrom']['Crossline'] = 'X'
Viewer['Viewer3D']['ViewFrom']['Z'] = 'Z'
#
Viewer['Player'] = {}
Viewer['Player']['First'] = 'A'
Viewer['Player']['Previous'] = 'S'
Viewer['Player']['Next'] = 'D'
Viewer['Player']['Last'] = 'F'
Viewer['Player']['Backward'] = 'Q'
Viewer['Player']['Forward'] = 'W'
Viewer['Player']['Pause'] = 'P'
Viewer['Player']['Interval'] = 1


def checkGUI(gui):
    """
    Check if the GUI setting good to use

    Args:
        gui:    GUI setting dictionary

    Return:
        True or false
    """

    if len(gui.keys()) < 1:
        return False
    if 'Toolbar' not in gui.keys():
        return False
    if len(gui['Toolbar'].keys()) < 1:
        return False
    if 'Left' not in gui['Toolbar'].keys() \
        or 'Right' not in gui['Toolbar'].keys() \
        or 'Top' not in gui['Toolbar'].keys() \
        or 'Bottom' not in gui['Toolbar'].keys():
        return False
    #
    return True


def checkGeneral(general):
    """
    Check if the general setting good to use

    Args:
        general:    general setting dictionary

    Return:
        True or false
    """

    if len(general.keys()) < 1:
        return False
    if 'RootPath' not in general.keys():
        return False
    if len(general['RootPath']) < 1:
        return False
    #
    return True


def checkVisual(visual):
    """
    Check if the Visual setting good to use

    Args:
        visual: Visual setting dictionary

    Return:
        True or false
    """

    if len(visual.keys()) < 1:
        return False
    #
    if 'Font' not in visual.keys():
        return False
    if len(visual['Font'].keys()) < 1:
        return False
    if 'Name' not in visual['Font'].keys() \
        or 'Color' not in visual['Font'].keys() \
        or 'Style' not in visual['Font'].keys() \
        or 'Weight' not in visual['Font'].keys() \
        or 'Size' not in visual['Font'].keys():
        return False
    #
    if 'Line' not in visual.keys():
        return False
    if len(visual['Line'].keys()) < 1:
        return False
    if 'Color' not in visual['Line'].keys() \
        or 'Width' not in visual['Line'].keys() \
        or 'Style' not in visual['Line'].keys() \
        or 'MarkerStyle' not in visual['Line'].keys() \
        or 'MarkerSize' not in visual['Line'].keys():
        return False
    #
    if 'Image' not in visual.keys():
        return False
    if len(visual['Image'].keys()) < 1:
        return False
    if 'Colormap' not in visual['Image'].keys():
        return False
    if 'Interpolation' not in visual['Image'].keys():
        return False
    #
    return True


def checkViewer(viewer):
    """
    Check if the Viewer setting good to use

    Args:
        viewer: Viewer setting dictionary

    Return:
        True or false
    """

    if len(viewer.keys()) < 1:
        return False
    #
    if 'Viewer3D' not in viewer.keys():
        return False
    if len(viewer['Viewer3D'].keys()) < 1:
        return False
    if 'GoHome' not in viewer['Viewer3D'].keys():
        return False
    if 'ViewFrom' not in viewer['Viewer3D'].keys():
        return False
    if len(viewer['Viewer3D']['ViewFrom'].keys()) < 1:
        return False
    if 'Inline' not in viewer['Viewer3D']['ViewFrom'].keys():
        return False
    if 'Crossline' not in viewer['Viewer3D']['ViewFrom'].keys():
        return False
    if 'Z' not in viewer['Viewer3D']['ViewFrom'].keys():
        return False
    #
    if 'Player' not in viewer.keys():
        return False
    if len(viewer['Player'].keys()) < 1:
        return False
    if 'First' not in viewer['Player'].keys():
        return False
    if 'Previous' not in viewer['Player'].keys():
        return False
    if 'Next' not in viewer['Player'].keys():
        return False
    if 'Last' not in viewer['Player'].keys():
        return False
    if 'Backward' not in viewer['Player'].keys():
        return False
    if 'Forward' not in viewer['Player'].keys():
        return False
    if 'Pause' not in viewer['Player'].keys():
        return False
    if 'Interval' not in viewer['Player'].keys():
        return False
    #
    return True


def checkSettings(gui={}, general={}, visual={}, viewer={}):
    """
    Check if the Viewer setting good to use

    Args:
        viewer: Viewer setting dictionary

    Return:
        True or false
    """

    if checkGUI(gui) is False \
            or checkGeneral(general) is False \
            or checkVisual(visual) is False \
            or checkViewer(viewer) is False:
        return False
    #
    return True


class settings:
    # Pack all functions as a class
    GUI = GUI
    General = General
    Visual = Visual
    Viewer = Viewer
    #
    checkGUI = checkGUI
    checkGeneral = checkGeneral
    checkVisual = checkVisual
    checkViewer = checkViewer
    #
    Settings = {}
    Settings['General'] = General
    Settings['Gui'] = GUI
    Settings['Visual'] = Visual
    Settings['Viewer'] = Viewer
    #
    checkSettings = checkSettings




#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

from __future__ import absolute_import, division, print_function

__all__ = ['keyboard', 'settings']


import os, sys
sys.path.append(os.path.dirname(__file__)[:-5][:-4][:-13])
#
from cognitivegeo.src.core.keyboard import keyboard as keyboard
from cognitivegeo.src.core.settings import settings as settings


from abc import abstractmethod, ABCMeta
from collections import Mapping, Sequence, OrderedDict
from fractions import Fraction
import reprlib
import os, sys
sys.path.append(os.path.dirname(__file__)[:-6][:-4][:-13])
from cognitivegeo.src.segpy.util import contains_duplicates, measure_stride, minmax


class CatalogBuilder(object):
    """Use a catalog builder to construct optimised, immutable mappings.

    A CatalogBuilder is useful when, depending on the particular keys and
    values used, a more compact or efficient representation of the mapping
    is possible than, say, a regular dictionary. The CatalogBuilder
    accumulates values and then, once all values have been added, analyzes
    the keys and values to produce a more optimized representation of the
    mapping.
    """

    def __init__(self, mapping=None):
        """Initialize a Catalog Builder.

        Args:
            mapping: An optional mapping (such as a dictionary) of items.
        """
        self._catalog = []
        if mapping is not None:
            for key, value in mapping.items():
                self.add(key, value)

    def add(self, index, value):
        """Add an item.

        Each index must be unique if create() is to be subsequently
        called successfully, although duplicate index values will be
        accepted by this call without complaint.

        """
        self._catalog.append((index, value))

    def create(self):
        """Create a possibly more optimized representation of the mapping.

        In this worst case, this method returns an object which is
        essentially an immutable dictionary. In the best case, the
        space savings can be vast.

        Returns:
            A mapping, if a unique mapping from indexes to values is
            possible, otherwise None.

        """

        # This method examines the contents of the mapping using
        # various heuristics to come up with a better representation.

        if len(self._catalog) < 2:
            return DictionaryCatalog(self._catalog)

        # In-place sort by index
        self._catalog.sort(key=lambda index_value: index_value[0])

        if contains_duplicates(index for index, value in self._catalog):
            return None

        if all(isinstance(index, Sequence) and (len(index) == 2)
               for index, value in self._catalog):
            return self._create_catalog_2()

        return self._create_catalog_1()

    def _create_catalog_1(self):
        """Create a catalog for one-dimensional integer keys (i.e. scalars)
        """
        index_min = self._catalog[0][0]
        index_max = self._catalog[-1][0]
        index_stride = measure_stride(index for index, value in self._catalog)

        if index_stride is None:
            # Dictionary strategy - arbitrary keys and values
            return DictionaryCatalog(self._catalog)

        self._catalog.sort(key=lambda index_value: index_value[1])
        value_min = self._catalog[0][1]
        value_max = self._catalog[-1][1]
        value_stride = measure_stride(value for index, value in self._catalog)

        if index_stride is not None and value_stride == 0:
            assert value_min == value_max
            return RegularConstantCatalog(index_min,
                                          index_max,
                                          index_stride,
                                          value_min)

        if index_stride is None and value_stride == 0:
            assert value_min == value_max
            return ConstantCatalog(
                    (index for index, value in self._catalog),
                    value_min)

        if index_stride is not None and value_stride is None:
            # Regular index - regular keys and arbitrary values
            return RegularCatalog(index_min,
                                  index_max,
                                  index_stride,
                                  (value for index, value in self._catalog))

        assert (index_stride is not None) and (value_stride is not None)
        catalog = LinearRegularCatalog(index_min,
                                       index_max,
                                       index_stride,
                                       value_min,
                                       value_max,
                                       value_stride)
        return catalog

    def _create_catalog_2(self):
        """Create a catalog for two-dimensional integer keys.

        Each key must be a two-element sequence.
        """
        i_min, i_max = minmax(i for (i, j), value in self._catalog)
        j_min, j_max = minmax(j for (i, j), value in self._catalog)

        is_rm, diff = self._is_row_major(i_min, j_min, j_max)
        if is_rm:
            return RowMajorCatalog(i_min, i_max, j_min, j_max, diff)
        return DictionaryCatalog(self._catalog)

    def _is_row_major(self, i_min, j_min, j_max):
        """Does row major ordering predict values from keys?

        In row-major order the last dimension is contiguous, and so changes
        quickest, when moving through the elements in storage order. Hence
        the number of rows is the number of distinct i values and the numbers
        of elements in each row (i.e. columns) is the number of distinct j.

        Args:
            i_min: The minimum i value.
            j_min: The minimum j value.
            j_max: The maximum j value.

        Returns:
            A 2-tuple containing, in the first element True if the values can
            be predicted from the keys by assuming a row-major ordering,
            otherwise False. If True, the second element will be a constant
            offset, otherwise it can be ignored.
        """
        diff = None
        for (i, j), actual_value in self._catalog:
            proposed_value = (i - i_min) * (j_max + 1 - j_min) + (j - j_min)
            current_diff = actual_value - proposed_value
            if diff is None:
                diff = current_diff
            if current_diff != diff:
                return False, None
        return True, diff


class Catalog(Mapping):
    """An abstract base class for Catalogs which provides min and max keys and values."""
    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self, key_min=None, key_max=None, value_min=None, value_max=None):
        """Must be overridden and called by subclasses.

        Args:
            key_min: Optional minimum key.
            key_max: Optional maximum key.
            value_min: Optional minimum value.
            value_max: Optional maximum value.
        """
        self._min_key = key_min
        self._max_key = key_max
        self._min_value = value_min
        self._max_value = value_max

    def key_min(self):
        """Minimum key"""
        if self._min_key is None:
            self._min_key = min(self.keys())
        return self._min_key

    def key_max(self):
        """Maximum key"""
        if self._max_key is None:
            self._max_key = max(self.keys())
        return self._max_key

    def value_min(self):
        """Minimum value"""
        if self._min_value is None:
            self._min_value = min(self.values())
        return self._min_value

    def value_max(self):
        """Maximum value"""
        if self._max_value is None:
            self._max_value = max(self.values())
        return self._max_value


class RowMajorCatalog(Catalog):
    """A mapping which assumes a row-major ordering of a two-dimensional matrix.

    This is the ordering of items in a two-dimensional matrix where in
    the (i, j) key tuple the j value changes fastest when iterating
    through the items in order.

    A RowMajorCatalog predicts the value v from the key (i, j) according to the
    following formula:

        v = (i - i_min) * j_max + (j - j_min) + c

    for
        i_min <= i <= i_max
        j_min <= j <= j_max

    and where c is an integer constant to allow zero- or one-based indexing.
    """

    def __init__(self, i_min, i_max, j_min, j_max, c):
        """Initialize a RowMajorCatalog.

        Args:
            i_min (int): The minimum i value.
            i_max (int): The maximum i value.
            j_min (int): The minimum j value.
            j_max (int): The maximum j value.
            c (int): The constant offset
        """
        super(RowMajorCatalog, self).__init__()
        self._i_min = i_min
        self._i_max = i_max
        self._j_min = j_min
        self._j_max = j_max
        self._c = c

    @property
    def i_min(self):
        """Minimum i value"""
        return self._i_min

    @property
    def i_max(self):
        """Maximum i value"""
        return self._i_max

    @property
    def j_min(self):
        """Minimum j value"""
        return self._j_min

    @property
    def j_max(self):
        """Maximum j value"""
        return self._j_max

    def key_min(self):
        """Minimum (i, j) key"""
        return self._i_min, self._j_min

    def key_max(self):
        """Maximum (i, j) key"""
        return self._i_max, self._j_max

    def value_min(self):
        """Minimum value at key_min"""
        return self[self.key_min()]

    def value_max(self):
        """Maximum value at key_max"""
        return self[self.key_max()]

    def __getitem__(self, key):
        i, j = key
        if not (self._i_min <= i <= self._i_max) and \
               (self._j_min <= j <= self._j_max):
            raise KeyError("{!r} key {!r} out of range".format(self, key))
        value = (i - self._i_min) * (self._j_max + 1 - self._j_min) + (j - self._j_min) + self._c
        return value

    def __contains__(self, key):
        i, j = key
        return (self._i_min <= i <= self._i_max) and \
               (self._j_min <= j <= self._j_max)

    def __len__(self):
        return (self._i_max - self._i_min) * (self._j_max + 1 - self._j_min)

    def __iter__(self):
        for i in range(self._i_min, self._i_max + 1):
            for j in range(self._j_min, self._j_max + 1):
                yield (i, j)

    def __repr__(self):
        return '{}({}, {}, {}, {}, {})'.format(
            self.__class__.__name__,
            self._i_min, self._i_max, self._j_min, self._j_max, self._c)


class DictionaryCatalog(Catalog):
    """An immutable, ordered, dictionary mapping.
    """

    def __init__(self, items):
        super(DictionaryCatalog, self).__init__()
        self._items = OrderedDict(items)

    def __getitem__(self, key):
        return self._items[key]

    def __iter__(self):
        return iter(self._items)

    def __len__(self):
        return len(self._items)

    def __contains__(self, item):
        return item in self._items

    def __repr__(self):
        return '{}({})'.format(
            self.__class__.__name__, reprlib.repr(self._items.items()))


class RegularConstantCatalog(Catalog):
    """Mapping with keys ordered with regular spacing along the number line.

    The values associated with the keys are constant.
    """

    def __init__(self, key_min, key_max, key_stride, value):
        """Initialize a RegularConstantCatalog.

        The catalog is initialized by a description of how the keys
        are distributed along the number line, and a value which
        corresponds with all keys.

        Args:
            key_min: The minimum key.
            key_max: The maximum key.
            key_stride: The difference between successive keys.
            value: A value associated with all keys.
        """
        key_range = key_max - key_min
        if key_range % key_stride != 0:
            raise ValueError("RegularIndex key range {!r} is not "
                             "a multiple of stride {!r}".format(
                                 key_stride, key_range))
        super(RegularConstantCatalog, self).__init__(
            key_min=key_min,
            key_max=key_max,
            value_min=value,
            value_max=value)
        self._key_stride = key_stride

    def __getitem__(self, key):
        if key not in self:
            raise KeyError("{!r} does not contain key {!r}".format(self, key))
        return self.value_min()

    def __len__(self):
        return 1 + (self.key_max() - self.key_min()) / self._key_stride

    def __contains__(self, key):
        return (self.key_min() <= key <= self.key_max()) and \
               ((key - self.key_min()) % self._key_stride == 0)

    def __iter__(self):
        return iter(range(self.key_min(),
                          self.key_max() + 1,
                          self._key_stride))

    def __repr__(self):
        return '{}({}, {}, {}, {})'.format(
            self.__class__.__name__,
            self.key_min(),
            self.key_max(),
            self._key_stride,
            self.value_min())


class ConstantCatalog(Catalog):
    """Mapping with arbitrary keys and a single constant value.
    """

    def __init__(self, keys, value):
        """Initialize a RegularConstantCatalog.

        The catalog is initialized by a description with an iterable series of
        keys and a constant value to be associated with all the keys.

        Args:
            keys: An iterable series of distinct keys.
            key_max: The maximum key.
            key_stride: The difference between successive keys.
            value: A value associated with all keys.
        """
        super(ConstantCatalog, self).__init__(value_min=value, value_max=value)
        self._items = frozenset(keys)

    def __getitem__(self, key):
        if key not in self:
            raise KeyError("{!r} does not contain key {!r}".format(self, key))
        return self.value_min()

    def __len__(self):
        return len(self._items)

    def __contains__(self, key):
        return key in self._items

    def __iter__(self):
        return iter(self._items)

    def __repr__(self):
        return '{}({}, {})'.format(
            self.__class__.__name__,
            reprlib.repr(self._items),
            self.value_min())


class RegularCatalog(Catalog):
    """Mapping with keys ordered with regular spacing along the number line.

    The values associated with the keys are arbitrary.
    """

    def __init__(self, key_min, key_max, key_stride, values):
        """Initialize a RegularCatalog.

        The catalog is initialized by a description of how the keys
        are distributed along the number line, and an iterable series of
        corresponding values.

        Args:
            key_min: The minimum key.
            key_max: The maximum key.
            key_stride: The difference between successive keys.
            values: An iterable series of values corresponding to the keys.

        Raises:
            ValueError: There is any inconsistency in the keys, stride,
                and/or values.
        """
        key_range = key_max - key_min
        if key_range % key_stride != 0:
            raise ValueError("{} key range {!r} is not "
                             "a multiple of stride {!r}".format(self.__class__.__name__,
                                                                key_stride, key_range))
        super(RegularCatalog, self).__init__(key_min=key_min, key_max=key_max)
        self._key_stride = key_stride
        self._values = list(values)
        num_keys = key_range // key_stride
        if num_keys != len(self._values):
            raise ValueError("{} key range and values inconsistent".format(self.__class__.__name__))

    def __getitem__(self, key):
        if not (self.key_min() <= key <= self.key_max()):
            raise KeyError("{!r} key {!r} out of range".format(self, key))
        offset = key - self.key_min()
        if offset % self._key_stride != 0:
            raise KeyError("{!r} does not contain key {!r}".format(self, key))
        index = offset // self._key_stride
        return self._values[index]

    def __len__(self):
        return len(self._values)

    def __contains__(self, key):
        return (self.key_min() <= key <= self.key_max()) and \
               ((key - self.key_min()) % self._key_stride == 0)

    def __iter__(self):
        return iter(range(self.key_min(),
                          self.key_max() + 1,
                          self._key_stride))

    def __repr__(self):
        return '{}({}, {}, {}, {})'.format(
            self.__class__.__name__,
            self.key_min(),
            self.key_max(),
            self._key_stride,
            reprlib.repr(self._values))


class LinearRegularCatalog(Catalog):
    """A mapping which assumes a linear relationship between keys and values.

    This is the ordering of items in a two-dimensional matrix where in
    the (i, j) key tuple the j value changes fastest when iterating
    through the items in order.

    A LinearRegularCatalog predicts the value v from the key according to the
    following formula:

        v = (value_max - value_min) / (key_max - key_min) * (key - key_min) + value_min
    """

    def __init__(self,
                 key_min,
                 key_max,
                 key_stride,
                 value_min,
                 value_max,
                 value_stride):
        """Initialize a LinearRegularCatalog.

        Args:
            key_min: The minimum key.
            key_max: The maximum key.
            key_stride: The difference between successive keys.
            value_min: The minimum value.
            value_max: The maximum value.

        Raises:
            ValueError: There is any inconsistency in the keys, strides,
                and/or values.
        """
        key_range = key_max - key_min
        if key_range % key_stride != 0:
            raise ValueError("{} key range {!r} is not "
                             "a multiple of key stride {!r}".format(
                                 self.__class__.__name__,
                                 key_stride,
                                 key_range))
        self._key_stride = key_stride

        value_range = value_max - value_min
        if value_range % value_stride != 0:
            raise ValueError("{} value range {!r} is not "
                             "a multiple of value stride {!r}".format(
                                 self.__class__.__name__,
                                 value_stride,
                                 value_range))
        self._value_stride = value_stride

        super(LinearRegularCatalog, self).__init__(
            key_min=key_min,
            key_max=key_max,
            value_min=value_min,
            value_max=value_max)

        num_keys = (self.key_max() - self.key_min()) // self._key_stride
        num_values = (self.value_max() - self.value_min()) // self._value_stride
        if num_keys != num_values:
            raise ValueError("{} inconsistent number of "
                             "keys {} and values {}".format(
                                 self.__class__.__name__,
                                 num_keys,
                                 num_values))

        self._m = Fraction(self.value_max() - self.value_min(),
                           self.key_max() - self.key_min())

    def __getitem__(self, key):
        if not (self.key_min() <= key <= self.key_max()):
            raise KeyError("{!r} key {!r} out of range".format(self, key))
        offset = key - self.key_min()
        if offset % self._key_stride != 0:
            raise KeyError("{!r} does not contain key {!r}".format(self, key))

        v = self._m * (key - self.key_min()) + self.value_min()
        assert v.denominator == 1
        return v.numerator

    def __len__(self):
        return 1 + (self.key_max() - self.key_min()) // self._key_stride

    def __contains__(self, key):
        return (self.key_min() <= key <= self.key_max()) and \
               ((key - self.key_min()) % self._key_stride == 0)

    def __iter__(self):
        return iter(range(self.key_min(), self.key_max() + 1, self._key_stride))

    def __repr__(self):
        return '{}({}, {}, {}, {}, {}, {})'.format(
            self.__class__.__name__,
            self.key_min(),
            self.key_max(),
            self._key_stride,
            self.value_min(),
            self.value_max(),
            self._value_stride)



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for exporting seismic images


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.seismic.visualization import visualization as seis_vis
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class exportseisimageset(object):

    survinfo = {}
    seisdata = {}
    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)
    dialog = None

    cmapsname = ['Seismic', 'Phase', 'Frequency', 'Red-White-Blue', 'Gray-Scale',
                 'Black-White-Red', 'Black-White-Green', 'Black-White-Blue',
                 'White-Red-Black', 'White-Green-Black', 'White-Blue-Black',
                 'Black-Red', 'Black-Green', 'Black-Blue']
    cmaps = ['seismic', 'phase', 'frequency', 'red_white_blue', 'white_gray_black',
             'black_white_red', 'black_white_green', 'black_white_blue',
             'white_red_black', 'white_green_black', 'white_blue_black',
             'black_red', 'black_green', 'black_blue']

    def setupGUI(self, ExportSeisImageSet):
        ExportSeisImageSet.setObjectName("ExportSeisImageSet")
        ExportSeisImageSet.setFixedSize(400, 550)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/image.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ExportSeisImageSet.setWindowIcon(icon)
        #
        self.lblattrib = QtWidgets.QLabel(ExportSeisImageSet)
        self.lblattrib.setObjectName("lblattrib")
        self.lblattrib.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.lwgattrib = QtWidgets.QListWidget(ExportSeisImageSet)
        self.lwgattrib.setObjectName("lwgattrib")
        self.lwgattrib.setGeometry(QtCore.QRect(160, 10, 230, 200))
        self.lwgattrib.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.lbltype = QtWidgets.QLabel(ExportSeisImageSet)
        self.lbltype.setObjectName("lbltype")
        self.lbltype.setGeometry(QtCore.QRect(10, 230, 150, 30))
        self.cbbtype = QtWidgets.QComboBox(ExportSeisImageSet)
        self.cbbtype.setObjectName("cbbtype")
        self.cbbtype.setGeometry(QtCore.QRect(160, 230, 230, 30))
        self.lblslice = QtWidgets.QLabel(ExportSeisImageSet)
        self.lblslice.setObjectName("lblslice")
        self.lblslice.setGeometry(QtCore.QRect(10, 280, 150, 30))
        self.rdbsliceall = QtWidgets.QRadioButton(ExportSeisImageSet)
        self.rdbsliceall.setObjectName("rdbsliceall")
        self.rdbsliceall.setGeometry(QtCore.QRect(160, 280, 230, 30))
        self.rdbslicedef = QtWidgets.QRadioButton(ExportSeisImageSet)
        self.rdbslicedef.setObjectName("rdbslicedef")
        self.rdbslicedef.setGeometry(QtCore.QRect(160, 320, 230, 30))
        self.ldtslicedef = QtWidgets.QLineEdit(ExportSeisImageSet)
        self.ldtslicedef.setObjectName("ldtslicedef")
        self.ldtslicedef.setGeometry(QtCore.QRect(160, 360, 230, 30))
        self.lblcmap = QtWidgets.QLabel(ExportSeisImageSet)
        self.lblcmap.setObjectName("lblcmap")
        self.lblcmap.setGeometry(QtCore.QRect(10, 400, 150, 30))
        self.cbbcmap = QtWidgets.QComboBox(ExportSeisImageSet)
        self.cbbcmap.setObjectName("cbbcmap")
        self.cbbcmap.setGeometry(QtCore.QRect(160, 400, 170, 30))
        self.cbxflip = QtWidgets.QCheckBox(ExportSeisImageSet)
        self.cbxflip.setObjectName("cbxflip")
        self.cbxflip.setGeometry(QtCore.QRect(340, 400, 50, 30))
        self.lblsave = QtWidgets.QLabel(ExportSeisImageSet)
        self.lblsave.setObjectName("lblsave")
        self.lblsave.setGeometry(QtCore.QRect(10, 440, 50, 30))
        self.ldtsave = QtWidgets.QLineEdit(ExportSeisImageSet)
        self.ldtsave.setObjectName("ldtsave")
        self.ldtsave.setGeometry(QtCore.QRect(70, 440, 250, 30))
        self.btnsave = QtWidgets.QPushButton(ExportSeisImageSet)
        self.btnsave.setObjectName("btnsave")
        self.btnsave.setGeometry(QtCore.QRect(330, 440, 60, 30))
        self.btnexportimage = QtWidgets.QPushButton(ExportSeisImageSet)
        self.btnexportimage.setObjectName("btnexportimage")
        self.btnexportimage.setGeometry(QtCore.QRect(120, 490, 160, 30))
        self.btnexportimage.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ExportSeisImageSet)
        self.msgbox.setObjectName("msgbox")
        _center_x = ExportSeisImageSet.geometry().center().x()
        _center_y = ExportSeisImageSet.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ExportSeisImageSet)
        QtCore.QMetaObject.connectSlotsByName(ExportSeisImageSet)


    def retranslateGUI(self, ExportSeisImageSet):
        self.dialog = ExportSeisImageSet
        #
        _translate = QtCore.QCoreApplication.translate
        ExportSeisImageSet.setWindowTitle(_translate("ExportSeisImageSet", "Export Seismic ImageSet"))
        self.lblattrib.setText(_translate("ExportSeisImageSet", "Select output properties:"))
        if self.checkSurvInfo() is True:
            _firstattrib = None
            for i in sorted(self.seisdata.keys()):
                if self.checkSeisData(i):
                    item = QtWidgets.QListWidgetItem(self.lwgattrib)
                    item.setText(_translate("ExportSeisImageSet", i))
                    self.lwgattrib.addItem(item)
                    if _firstattrib is None:
                        _firstattrib = item
            self.lwgattrib.setCurrentItem(_firstattrib)
        self.lbltype.setText(_translate("ExportSeisImageSet", "Select orientation:"))
        self.cbbtype.addItems(['Inline', 'Crossline', 'Time/depth'])
        self.cbbtype.setItemIcon(0, QtGui.QIcon(os.path.join(self.iconpath, "icons/visinl.png")))
        self.cbbtype.setItemIcon(1, QtGui.QIcon(os.path.join(self.iconpath, "icons/visinl.png")))
        self.cbbtype.setItemIcon(2, QtGui.QIcon(os.path.join(self.iconpath, "icons/visz.png")))
        self.lblslice.setText(_translate("ExportSeisImageSet", "Sepcify output slices:"))
        self.rdbsliceall.setText(_translate("ExportSeisImageSet", "All within input dimensions"))
        self.rdbsliceall.setChecked(True)
        self.rdbsliceall.clicked.connect(self.clickRdbSliceAll)
        self.rdbslicedef.setText(_translate("ExportSeisImageSet", "User-defined (separated by ',')"))
        self.rdbslicedef.setChecked(False)
        self.rdbslicedef.clicked.connect(self.clickRdbSliceDef)
        self.ldtslicedef.setText(_translate("ExportSeisImageSet", "0"))
        self.ldtslicedef.setEnabled(False)
        self.lblcmap.setText(_translate("Plot2DXlSlice", "\t    Color map:"))
        self.cbbcmap.addItems(self.cmapsname)
        for _i in range(len(self.cmaps)):
            self.cbbcmap.setItemIcon(_i, QtGui.QIcon(
                QtGui.QPixmap(os.path.join(self.iconpath, "icons/cmap_" + self.cmaps[_i] + ".png")).scaled(80, 30)))
        self.cbxflip.setText(_translate("ExportSeisImageSet", "Flip"))
        self.lblsave.setText(_translate("ExportSeisImageSet", "Save to:"))
        self.ldtsave.setText(_translate("ExportSeisImageSet", os.path.abspath(self.rootpath)))
        self.btnsave.setText(_translate("ExportSeisImageSet", "Browse"))
        self.btnsave.clicked.connect(self.clickBtnSave)
        self.btnexportimage.setText(_translate("ExportSeisImageSet", "Export Seismic ImageSet"))
        self.btnexportimage.clicked.connect(self.clickBtnExportSeisImageSet)


    def clickRdbSliceAll(self):
        if self.rdbsliceall.isChecked():
            self.rdbsliceall.setChecked(True)
            self.rdbslicedef.setChecked(False)
            self.ldtslicedef.setText("0")
            self.ldtslicedef.setEnabled(False)


    def clickRdbSliceDef(self):
        if self.rdbslicedef.isChecked():
            self.rdbslicedef.setChecked(True)
            self.rdbsliceall.setChecked(False)
            self.ldtslicedef.setEnabled(True)


    def clickBtnSave(self):
        _dialog = QtWidgets.QFileDialog()
        _file = _dialog.getExistingDirectory(None, 'Select Export Folder', self.rootpath,
                                             options=QtWidgets.QFileDialog.DontUseNativeDialog)
        if len(_file) > 0:
            self.ldtsave.setText(_file)


    def clickBtnExportSeisImageSet(self):
        self.refreshMsgBox()
        #
        _attriblist = self.lwgattrib.selectedItems()
        if len(_attriblist) < 1:
            vis_msg.print("ERROR in ExportSeisImageSet: No property selected for export", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Export Seismic ImageSet',
                                           'No property selected for export')
            return
        if self.rdbslicedef.isChecked():
            for s in self.ldtslicedef.text().split(','):
                if basic_data.str2int(s) is False:
                    vis_msg.print("ERROR in ExportSeisImageSet: Non-integer slice selection", type='error')
                    QtWidgets.QMessageBox.critical(self.msgbox,
                                                   'Export Seismic ImageSet',
                                                   'Non-integer slice selection')
                    return
        #
        # Progress dialog
        _pgsdlg = QtWidgets.QProgressDialog()
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/image.png")),
                       QtGui.QIcon.Normal, QtGui.QIcon.Off)
        _pgsdlg.setWindowIcon(icon)
        _pgsdlg.setWindowTitle('Export ' + str(len(_attriblist)) + ' Seismic Image')
        _pgsdlg.setCancelButton(None)
        _pgsdlg.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
        _pgsdlg.forceShow()
        _pgsdlg.setFixedWidth(400)
        # _pgsdlg.setMaximum(len(_attriblist))
        #
        _cmap = self.cbbcmap.currentIndex()
        _flip = self.cbxflip.isChecked()
        for i in range(len(_attriblist)):
            #
            # QtCore.QCoreApplication.instance().processEvents()
            # _pgsdlg.setValue(i)
            _pgsdlg.setWindowTitle('Export '+str(i+1)+' of '+str(len(_attriblist))+' Seismic ImageSet')
            #
            _sls = None
            if self.rdbslicedef.isChecked():
                _sls = [basic_data.str2int(s) for s in self.ldtslicedef.text().split(',')]
            _imagefile = os.path.join(self.ldtsave.text(),
                                      _attriblist[i].text() + '_')
            _data = self.seisdata[_attriblist[i].text()]
            print("ExportSeisImageSet: Export %d of %d ImageSet: %s" % (i + 1, len(_attriblist), _attriblist[i].text()))
            if self.cbbtype.currentIndex() == 0:
                seis_vis.saveSeisILSliceFrom3DMat(_data, _imagefile, inlsls=_sls, seisinfo=self.survinfo,
                                                  valuemin=np.min(_data),
                                                  valuemax=np.max(_data),
                                                  colormap=self.cmaps[_cmap],
                                                  flipcmap=_flip,
                                                  verbose=False
                                                  )
            if self.cbbtype.currentIndex() == 1:
                seis_vis.saveSeisXLSliceFrom3DMat(_data, _imagefile, xlsls=_sls, seisinfo=self.survinfo,
                                                 valuemin=np.min(_data),
                                                 valuemax=np.max(_data),
                                                 colormap=self.cmaps[_cmap],
                                                 flipcmap=_flip,
                                                 verbose=False, qpgsdlg=_pgsdlg)
            if self.cbbtype.currentIndex() == 2:
                seis_vis.saveSeisZSliceFrom3DMat(_data, _imagefile, zsls=_sls, seisinfo=self.survinfo,
                                                valuemin=np.min(_data),
                                                valuemax=np.max(_data),
                                                colormap=self.cmaps[_cmap],
                                                flipcmap=_flip,
                                                verbose=False)
            #
        # _pgsdlg.setValue(len(_attriblist))
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Export Seismic ImageSet",
                                          str(len(_attriblist)) + " properties exported as ImageSet successfully")
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("ExportSeisImageSet: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Export Seismic ImageSet',
            #                                'Survey not found')
            return False
        return True


    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)



if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ExportSeisImageSet = QtWidgets.QWidget()
    gui = exportseisimageset()
    gui.setupGUI(ExportSeisImageSet)
    ExportSeisImageSet.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     November 2019                                                                   #
#                                                                                           #
#############################################################################################

# basic functions for 3d viewer

import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.core.keyboard import keyboard as core_key


__all__ = ['viewer3d']

GoHomeKeyList = core_key.LetterKeyList

ViewFromPropertyList = ['Inline', 'Crossline', 'Z']
ViewFromKeyList = core_key.LetterKeyList


class viewer3d:
    # Pack all functions as a class
    #
    GoHomeKeyList = GoHomeKeyList
    #
    ViewFromPropertyList = ViewFromPropertyList
    ViewFromKeyList = ViewFromKeyList


from collections import OrderedDict
from weakref import WeakKeyDictionary
from itertools import chain
import os, sys
sys.path.append(os.path.dirname(__file__)[:-6][:-4][:-13])
from cognitivegeo.src.segpy.docstring import docstring_property
from cognitivegeo.src.segpy.util import underscores_to_camelcase, first_sentence, super_class


class Header:
    """An abstract base class for header format definitions."""

    def __init__(self, **kwargs):
        # TODO: This has terrible performance - better to handle validation optimistically by overriding __getattr__()
        for keyword, arg in kwargs.items():
            if keyword not in self.ordered_field_names():
                raise TypeError("{!r} is not a recognised field name for {!r}".format(keyword, self.__class__.__name__))
            setattr(self, keyword, arg)

    _ordered_field_names = tuple()

    @classmethod
    def ordered_field_names(cls):
        """The ordered list of field names.

        This is a metamethod which should be called on cls.

        Returns:
            An tuple containing the field names in order.
        """
        if cls is Header:
            return cls._ordered_field_names
        return super_class(cls).ordered_field_names() + cls._ordered_field_names


class FormatMeta(type):
    """A metaclass for header format classes.
    """

    @classmethod
    def __prepare__(mcs, name, bases):
        return OrderedDict()

    def __new__(mcs, name, bases, namespace):

        # TODO: This is a good point to validate that the fields are in order and that the
        # TODO: format specification is valid.  We shouldn't even build the class otherwise.

        # TODO: Also validate existence of LENGTH_IN_BYTES

        namespace['_ordered_field_names'] = tuple(name for name, attr in namespace.items()
                                                  if isinstance(attr, HeaderFieldDescriptor))

        transitive_bases = set(chain.from_iterable(type(base).mro(base) for base in bases))

        if Header not in transitive_bases:
            bases = (Header,) + bases

        for attr_name, attr in namespace.items():

            # This shenanigans is necessary so we can have all the following work is a useful way
            # help(class), help(instance), help(class.property) and help(instance.property)

            # Set the _name attribute of the field instance if it hasn't already been set
            if isinstance(attr, HeaderFieldDescriptor):
                if attr._name is None:
                    attr._name = attr_name

            # We rename the *class* and set its docstring so help() works usefully
            # when called with a class containing such fields.
            attr_class = attr.__class__
            if issubclass(attr_class, NamedField) and attr_class is not NamedField:
                attr_class.__name__ = underscores_to_camelcase(attr_name)
                attr_class.__doc__ = attr.documentation

        return super().__new__(mcs, name, bases, namespace)


class NamedField:
    """Instances of NamedField can be detected by the NamedDescriptorResolver metaclass."""

    def __init__(self, value_type, offset, default, documentation):
        self._name = None  # Set later by the metaclass
        self._value_type = value_type
        self._offset = int(offset)
        self._default = self._value_type(default)
        self._documentation = str(documentation)

    @property
    def name(self):
        "The field name."
        return self._name

    @property
    def value_type(self):
        "The field value type (e.g. Int32)"
        return self._value_type

    @property
    def offset(self):
        "The offset in bytes from the beginning of the header."
        return self._offset

    @property
    def default(self):
        "The default value of the field. Must be convertible to value_type."
        return self._default

    @property
    def documentation(self):
        "A descriptive text string."
        return self._documentation

    @docstring_property(__doc__)
    def __doc__(self):
        return first_sentence(self._documentation)

    def __repr__(self):
        return "{}(name={!r}, value_type={!r}, offset={!r}, default={!r})".format(
            self.__class__.__name__,
            self.name,
            self.value_type.__name__,
            self.offset,
            self.default)


def field(value_type, offset, default, documentation):
    """
    Args:
        value_type: The type of the field (e.g. Int32)

        offset: The offset in bytes for this field from the start of the header.

        default: The default value for this field.

        documentation: A docstring for the field. The first sentence should be usable
            as a brief description.

    Returns:
        An instance of a subclass of NamedField class.
    """

    # Create a class specifically for this field. This class will later get
    # renamed when the NamedDescriptorMangler metaclass does its job, to
    # a class name based on the field name.

    class SpecificField(HeaderFieldDescriptor):
        pass

    return SpecificField(value_type, offset, default, documentation)


class HeaderFieldDescriptor:

    def __init__(self, value_type, offset, default, documentation):
        self._named_field = NamedField(value_type, offset, default, documentation)
        self._instance_data = WeakKeyDictionary()

    @property
    def _name(self):
        return self._named_field.name

    @_name.setter
    def _name(self, value):
        self._named_field._name = value

    def __get__(self, instance, owner):
        """Retrieve the format or instance data.

        When called on the class we return a NamedField instance containing the format data. For example:

            line_seq_num_default = TraceHeaderRev1.line_sequence_num.default
            line_seq_num_offset = TraceHeaderRev1.line_sequence_num.offset

        When called on an instance we return the field value.

            line_seq_num = my_trace_header.line_sequence_num
        """
        #print("HeaderFieldDescriptor.__get__({!r}, {!r}, {!r})".format(self, instance, owner))
        if instance is None:
            return self._named_field
        if instance not in self._instance_data:
            return self._named_field.default
        return self._instance_data[instance]

    def __set__(self, instance, value):
        """Set the field value."""
        try:
            self._instance_data[instance] = self._named_field._value_type(value)
        except ValueError as e:
            raise ValueError("Assigned value {!r} for {} attribute must be convertible to {}: {}"
                             .format(value, self._name, self._named_field._value_type.__name__, e)) from e

    def __delete__(self, instance):
        raise AttributeError("Can't delete {} attribute".format(self._name))

    @docstring_property(__doc__)
    def __doc__(self):
        return self._named_field._documentation

    # TODO: Get documentation of these descriptors working correctly
